; ###############################################################
;  			Tattoo-Maschinen-Steuerung
;			PIC 16F876
;	Geaendert:	KH Domnick
; ###############################################################

		;0123456789ABCDEF Stellen
#DEFINE	Version	"Tattoo Vers 1.1 "	; <=== hier die Version aendern
		;tt.mm.jj
#DEFINE	Datum	"11.10.08"		; <=== hier das Datum aendern

; ###############################################################

	list P=16f876, E=1, R=DEC	; Prozessortyp, Errorlevel und Zahlenformat
	include "p16f876.inc"		; Registerdefinitionen einbinden

	errorlevel	1

	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF
;;	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF & _CP_ALL

Debug		set	0	; 0 = Debug aus
				; 1 = Debug ein

Quarz		set	12	; 20 = 20,0000 MHz
				; 16 = 16,0000 MHz
				; 12 = 12,0000 MHz
				; 10 = 10,2400 MHz
				; 5 =  4,9152 MHz *9600 Baud
				; 4 =  4,0960 MHz
				; 2 =  2,4576 MHz

; ***************************************************************
; Definitionen
; ***************************************************************
		;	PORTA,0...2	; AN0-2	Poti 1...3
		;	PORTA,3		; AN3	Spannung
#DEFINE	oDigPot		PORTA,4		; CS Digital-Poti
		;	PORTA,5		; AN4	Strom

#DEFINE	iImax		PORTB,0		; Eingang Ueberstrom
#DEFINE	oPwm		PORTB,1		; Ausgang PWM
#DEFINE	iFuKo		PORTB,2		; Eingang Fuss-Kontakt
#DEFINE	oLed9		PORTB,3		; Ausgang LED 9 Ueberstrom
#DEFINE	oLed8		PORTB,4		; Ausgang LED 8 Ein / Quittierung
		;	PORTB,5...7	; Adresse LEDs 1...7

#DEFINE	iTast1		PORTC,0		; Eingang Taster 1
#DEFINE	iTast2		PORTC,1		; Eingang Taster 2
#DEFINE	iTast3		PORTC,2		; Eingang Taster 3
		;	PORTC,3...5	; SPI Digital-Poti
		;	PORTC,6...7	; RS232

; ***************************************************************
; Konstanten
; ***************************************************************
cr		EQU	0x0D	; Return
lf		EQU	0x0A	; LineFeed
hk		EQU	0x22	; Hochkomma
klauf		EQU	"<"
klzu		EQU	">"

kPtime		EQU	4	;   4 * 10ms =  40ms Prellzeit
kTlang		EQU	80	;  80 * 10ms = 800ms Tastendruck lang
kFlang		EQU	40	;  40 * 10ms = 400ms Fuss-Kontakt lang

; ***************************************************************
; Makros
; ***************************************************************
		; ---------------------------
		; Makro: Umschalten nach Rom-Page X
		; ---------------------------
M_Page0		macro
		bcf	PCLATH,3	; Rom-Page 0
		bcf	PCLATH,4
		endm

M_Page1		macro
		bsf	PCLATH,3	; Rom-Page 1
		bcf	PCLATH,4
		endm

M_Page2		macro
		bcf	PCLATH,3	; Rom-Page 2
		bsf	PCLATH,4
		endm

M_Page3		macro
		bsf	PCLATH,3	; Rom-Page 3
		bsf	PCLATH,4
		endm

		; ---------------------------
		; Makro: Umschalten auf RegisterBank X
		; ---------------------------
M_ClSt_Bank0	macro
		clrf	STATUS		; Bank 0
		endm

M_Bank0		macro
		bcf	STATUS,RP0	; Bank 0
		bcf	STATUS,RP1
		endm

M_Bank1		macro
		bsf	STATUS,RP0	; Bank 1
		bcf	STATUS,RP1
		endm

M_Bank2		macro
		bcf	STATUS,RP0	; Bank 2
		bsf	STATUS,RP1
		endm

M_Bank3		macro
		bsf	STATUS,RP0	; Bank 3
		bsf	STATUS,RP1
		endm

		; ---------------------------
		; Makro: MOV FileReg to FileReg
		; ---------------------------
M_movff		macro	VonFileReg,NachFileReg
		movfw	VonFileReg
		movwf	NachFileReg
		endm

		; ---------------------------
		; Makro: MOV Literal to FileReg
		; ---------------------------
M_movlf		macro	Literal,NachFileReg
		movlw	Literal
		movwf	NachFileReg
		endm

		; ---------------------------
		; Makro: MOV FileReg Bank 1 to W
		; ---------------------------
M_movb1w	macro	VonFileReg
		bsf	STATUS,RP0	; Bank 1
		movfw	VonFileReg
		bcf	STATUS,RP0	; Bank 0
		endm

		; ---------------------------
		; Makro: MOV W to FileReg Bank 1
		; ---------------------------
M_movwb1	macro	NachFileReg
		bsf	STATUS,RP0	; Bank 1
		movwf	NachFileReg
		bcf	STATUS,RP0	; Bank 0
		endm

		; ---------------------------
		; Makro: SUB W from FileReg Bank 1 to W
		; ---------------------------
M_subwb1	macro	VonFileReg
		bsf	STATUS,RP0	; Bank 1
		subwf	VonFileReg,W
		bcf	STATUS,RP0	; Bank 0
		endm

; ***************************************************************
; Programm-Definitionen
; ***************************************************************
#DEFINE	D_GlobIntEin	bsf	INTCON,GIE
#DEFINE	D_GlobIntAus	bcf	INTCON,GIE
#DEFINE	D_PeriIntEin	bsf	INTCON,PEIE
#DEFINE	D_PeriIntAus	bcf	INTCON,PEIE

#DEFINE	_C		STATUS,0
#DEFINE	_Z		STATUS,2
#DEFINE	skip_0		btfsc
#DEFINE	skip_1		btfss
#DEFINE	skip_NC		btfsc	STATUS,C	; oder skpNC
#DEFINE	skip_C		btfss	STATUS,C	; oder skpC
#DEFINE	skip_NZ		btfsc	STATUS,Z	; oder skpNZ
#DEFINE	skip_Z		btfss	STATUS,Z	; oder skpZ

; ***************************************************************
; Register in Bank 0
; ***************************************************************
RamBeg0		EQU	0x20	; Beginn Ram-Bereich

		; ---------------------------
		; Register fuer Mathe-Routinen
		; ---------------------------
MathErg0	EQU	0x20	; Ergebnis (16 Bit)
MathErg1	EQU	0x21
MathArgA0	EQU	0x22	; Argument A (24 Bit)
MathArgA1	EQU	0x23
MathArgA2	EQU	0x24
MathArgB0	EQU	0x25	; Argument B (16 Bit)
MathArgB1	EQU	0x26
MathTmp0	EQU	0x27	; Temporaer oder Rest
MathTmp1	EQU	0x28
MathZlr0	EQU	0x29	; Zaehler

		; ---------------------------
		; Register fuer Zaehler- und Timer
		; ---------------------------
z01ms		EQU	0x2A	; Dec-Zaehler 0,1 ms-Takt
z1ms		EQU	0x2B	; Dec-Zaehler   1 ms-Takt
z10ms		EQU	0x2C	; Dec-Zaehler  10 ms-Takt
z100ms		EQU	0x2D	; Dec-Zaehler 100 ms-Takt

t10ms		EQU	0x2E	; Dec-Timer  10 ms
t100ms		EQU	0x2F	; Dec-Timer 100 ms

zFreq		EQU	0x30	; Dec-Zaehler Frequenz		0,1 ms-Takt
zDuty		EQU	0x31	; Dec-Zaehler Duty		0,1 ms-Takt
zImax		EQU	0x32	; Inc-Zaehler Ueberstrom	0,1 ms-Takt
tImax		EQU	0x33	; Dec-Timer Ueberstrom		100 ms-Takt
zUpdIS		EQU	0x34	; Inc-Zaehler Ist/Soll Update
tTakt		EQU	0x35	; Dec-Timer TaktBetrieb		10 ms-Takt


zTast1		EQU	0x37	; Prell-Zaehler Taster 1
zTast2		EQU	0x38	; Prell-Zaehler Taster 2
zTast3		EQU	0x39	; Prell-Zaehler Taster 3
zFussK		EQU	0x3A	; Prell-Zaehler Fuss-Kontakt

		; ---------------------------
		; Register fuer AD-Werte
		; ---------------------------
aAdw_1		EQU	0x3B	; AD-Wert Poti 1
aAdw_2		EQU	0x3C	; AD-Wert Poti 2
aAdw_3		EQU	0x3D	; AD-Wert Poti 3
aAdw_4		EQU	0x3E	; AD-Wert Strom
aAdw_5		EQU	0x3F	; AD-Wert Spannung

		; ---------------------------
		; Register fuer Man und Soll
		; Reihenfolge mu mit pXXX uebereinstimmen
		; ---------------------------
mSpan		EQU	0x40	; Man.-Wert Volt
mFreq		EQU	0x41	; Man.-Wert Frequenz
mDuty		EQU	0x42	; Man.-Wert DutyCycle
mTakt		EQU	0x43	; Man.-Wert Takt-Zeit
mRuhe		EQU	0x44	; Man.-Wert Ruhe-Zeit
mBits		EQU	0x45	; Bits
#DEFINE	F_Takt	mBits,0		; 1=Taktbetrieb Ein

aWahl		EQU	0x47	; Akt Wahl

sSpan		EQU	0x48	; Soll-Wert Volt
sFreq		EQU	0x49	; Soll-Wert Frequenz
sDuty		EQU	0x4A	; Soll-Wert DutyCycle
sTakt		EQU	0x4B	; Soll-Wert Takt-Zeit
sRuhe		EQU	0x4C	; Soll-Wert Ruhe-Zeit

iSpan		EQU	0x4D	; Ist-Wert Volt
iFreq		EQU	0x4E	; Ist-Wert Frequenz
iDuty		EQU	0x4F	; Ist-Wert DutyCycle

		; ---------------------------
		; Aktuelle Parameter / Daten (max 8 Parameter)
		; ---------------------------
RamParam2	EQU	0x50
pGeschw		EQU	0x50	; Geschw. Annaeherung Ist - Soll
pImax		EQU	0x51	; Ueberstrom-Zeit
pModFk		EQU	0x52	; Modus Fuss-Kontakt
pVoltR		EQU	0x53	; Volt von Speicher / Regler

		; ---------------------------
		; Diverse Flags
		; ---------------------------
FlagsSt		EQU	0x58	; Flags Steuerung
#DEFINE	F_Start	FlagsSt,0	; 1=PWM ein
#DEFINE	F_Imax	FlagsSt,1	; 1=Ueberstrom
#DEFINE	F_Tein	FlagsSt,2	; 1=Takt Ein
#DEFINE	F_Bli1	FlagsSt,3	; Blinker 1	100ms
#DEFINE	F_Bli2	FlagsSt,4	; Blinker 2	200ms
#DEFINE	F_Bli8	FlagsSt,5	; Blinker 3	800ms
#DEFINE	F_UpdSI	FlagsSt,6	; 1=Soll/Ist aktualisieren
#DEFINE	F_Ta2X	FlagsSt,7	; Taste 2 + andere Taste

FlagsTa		EQU	0x59	; Flags Tasten
#DEFINE	F_Ta1K	FlagsTa,0	; Taste 1 kurz
#DEFINE	F_Ta1L	FlagsTa,1	; Taste 1 lang
#DEFINE	F_Ta2K	FlagsTa,2	; Taste 2 kurz
#DEFINE	F_Ta2L	FlagsTa,3	; Taste 2 lang
#DEFINE	F_Ta3K	FlagsTa,4	; Taste 3 kurz
#DEFINE	F_Ta3L	FlagsTa,5	; Taste 3 lang
#DEFINE	F_FuKoK	FlagsTa,6	; Fuss-Kontakt kurz
#DEFINE	F_FuKoL	FlagsTa,7	; Fuss-Kontakt lang

FlagsQ		EQU	0x5A	; Flags Quittierung
#DEFINE	F_QStor	FlagsQ,0	; Quittierung Store
#DEFINE	F_QKurz	FlagsQ,1	; Quittierung Kurzschlu

		; ---------------------------
		; Temporaere Register
		; ---------------------------
Temp0		EQU	0x60	; Temporaer (nicht fuer Interrupt)
Temp1		EQU	0x61
Temp2		EQU	0x62

Para0		EQU	0x63	; Temporaer Parameter
Para1		EQU	0x64
Para2		EQU	0x65

CmdByte		EQU	0x66	; Temporaer bei Cmd-Auswertung

		; ---------------------------
		; Zeiger-Register
		; ---------------------------
RxDOfs		EQU	0x67	; Offset RxD-RingPuffer
RxDPos		EQU	0x68	; Position RxD-RingPuffer
TxDOfs		EQU	0x69	; Offset TxD-RingPuffer
TxDPos		EQU	0x6A	; Position TxD-RingPuffer
TxDAnz		EQU	0x6B	; Anzahl Bytes im TxD-Puffer
CmdPos		EQU	0x6C	; Position Cmd-Puffer

RamEnd0		EQU	0x6D
RamAnz0		EQU	RamEnd0 - RamBeg0

ramP		EQU	0x6D	; Init P
ramI		EQU	0x6E	; Init I
ramC		EQU	0x6F	; Init C

		; ---------------------------
		; Diverse Register
		; Von Bank0...3 erreichbar (0x71...0x7F) !
		; ACHTUNG !!! 0x70 ist bei Verwendung von MPLAB ICD belegt
		; ---------------------------
MPLAB		EQU	0x70	; Fuer MPLAB 1
iBakW		EQU	0x71	; Interrupt Backup W-Register
iBakSTAT	EQU	0x72	; Interrupt Backup Status
iBakPCL		EQU	0x73	; Interrupt Backup Rom-Page (PCLATH)
iBakFSR		EQU	0x74	; Interrupt Backup Ind-Adresse
izUni		EQU	0x75	; Interrupt Universeller Zaehler

zUni		EQU	0x76	; Universeller Zaehler
AddrL		EQU	0x77	; Addresse Low  fuer Flash & Eeprom
AddrH		EQU	0x78	; Addresse High fuer Flash
EepByte		EQU	0x79	; Byte aus Flash & Eeprom

iTemp0		EQU	0x7A	; Temporaer im Interrupt
iTemp1		EQU	0x7B
iTemp2		EQU	0x7C
#DEFINE	F_iBit	iTemp2,0	; Temp. Bit im Interrupt

		; ---------------------------
		; Register fuer Flags
		; ---------------------------
Flags0		EQU	0x7D
#DEFINE	F_OK	Flags0,0	; Flag OK = 1 / nicht OK = 0
#DEFINE	F_Ret	Flags0,1	; Flag CR
#DEFINE	F_Semi	Flags0,2	; Flag Semikolon
#DEFINE	F_CmdE	Flags0,3	; Flag Cmd-Puffer Ende
#DEFINE	F_Pkt	Flags0,4	; 1=Punkt vor Einer-Stelle
#DEFINE	F_000	Flags0,5	; 1=fuehrende Nullen senden
#DEFINE	F_Ovfl	Flags0,6	; Flag berlauf
#DEFINE	F_Word	Flags0,7	; Flag Word (> 255)

Flags1		EQU	0x7E
#DEFINE	F_Fmt	Flags1,0	; 1=Ziffern formatiert senden
#DEFINE	F_Zif	Flags1,1	; 1=Ziffer 1...9 gesendet
#DEFINE	F_Eep	Flags1,2	; 1=Auch im Eeprom speichern
#DEFINE	F_Pkt2	Flags1,4	; 1=Punkt vor Zehner-Stelle

; ***************************************************************
; Register in Bank 1
; ***************************************************************
		; ---------------------------
		; Aktuelle Parameter / Daten (max 64 Parameter)
		; ---------------------------
RamParam1	EQU	0xA0
pSpanSp		EQU	0xA0	; 5 x Spannung	Speicher 1...5
pFreqSp		EQU	0xA5	; 5 x Frequenz
pDutySp		EQU	0xAA	; 5 x DutyCycle
pTaktSp		EQU	0xAF	; 5 x Taktbetrieb Takt-Zeit
pRuheSp		EQU	0xB4	; 5 x Taktbetrieb Ruhe-Zeit
pBitsSp		EQU	0xB9	; 5 x Bits
pRes1		EQU	0xBE
pRes2		EQU	0xBF
pSpanVon	EQU	0xC0	; Spannung	Poti Von
pSpanBis	EQU	0xC1	; Spannung	Poti Bis
pFreqVon	EQU	0xC2	; Frequenz	Poti Von
pFreqBis	EQU	0xC3	; Frequenz	Poti Bis
pDutyVon	EQU	0xC4	; DutyCycle	Poti Von
pDutyBis	EQU	0xC5	; DutyCycle	Poti Bis

; ***************************************************************
; Register in Bank 2
; ***************************************************************
		; ---------------------------
		; Kommando-Puffer fuer max 48 Byte
		; ---------------------------
CmdLen		EQU	48	; Puffer fuer 48 Byte
CmdPuf		EQU	0x110	; Adresse 0x110 bis 0x13F

		; ---------------------------
		; RingPuffer fuer RxD
		; ---------------------------
RxDLen		EQU	48	; Puffer fuer 48 Byte
RxDPuf		EQU	0x140	; Adresse 0x140 bis 0x16F

; ***************************************************************
; Register in Bank 3
; ***************************************************************
		; ---------------------------
		; RingPuffer fuer TxD
		; ---------------------------
TxDLen		EQU	96	; Puffer fuer 64 Byte
TxDPuf		EQU	0x190	; Adresse 0x190 bis 0x1EF

; ***************************************************************
; Eeprom-Register
; ***************************************************************
		; ---------------------------
		; Register ab Eeprom-Adr. 0x2100 max 256 Byte
		; ---------------------------

; ###############################################################
; Programmspeicher Page 0
; RESET: AufsprungAdr 0000h
; ###############################################################
		org	0x0000

Reset		clrf	PCLATH		; 00h Page 0
		goto	ProgBeg		; 01h Sprung zum ProgrammBeginn
		goto	Reset		; 02h Resetvektor
		goto	Reset		; 03h Resetvektor
	;;	goto	Interrupt	; 04h Interruptvektor
	;;	Kein GOTO !!!, falls nicht in Rom-Page 0 ausgeloest wurde

; ***************************************************************
; Interrupt-Serviceroutine ISR
; Ausloesung durch Receive
; Ausloesung durch Timer 2 / PR2
; 27 Befehle * 0,3333333 ys = 8,9999991 ys
; ***************************************************************
Interrupt	movwf	iBakW		; W retten nach iBakW
		swapf	STATUS,W	; Status mit swap nach W
		clrf	STATUS		; Bank 0
		movwf	iBakSTAT	; Status retten ohne Veraenderung

		movfw	PCLATH		; Rom-Page (PCLATH) retten
		movwf	iBakPCL
		clrf	PCLATH		; Auf Rom-Page 0 schalten

		movfw	FSR
		movwf	iBakFSR

		; ---------------------------
		; Die Isr wird erst verlassen, wenn auch die Interrupts
		; verarbeitet sind, deren xxxIF waehrend der Isr gesetzt wurde.
		; Weitere Backup und Restore von W, STATUS, etc. werden damit eingespart.
		; ---------------------------
Isr_TestIF	bcf	STATUS,RP0	; Bank 0, falls nach xxxIE-Abfrage auf Bank 1

		skip_0	PIR1,TMR2IF	; ... Timer 2 nicht
		goto	Isr_TMR2

		skip_0	PIR1,RCIF	; ... Receive nicht
		goto	Isr_RC

Isr_Ende	movfw	iBakFSR
		movwf	FSR

		movfw	iBakPCL		; Rom-Page (PCLATH) zurueck
		movwf	PCLATH

		swapf	iBakSTAT,W	; Statusregister und W zurueck
		movwf	STATUS
		swapf	iBakW
		swapf	iBakW,W
		retfie

; ===============================================================
; ISR Receive (Sio RxD)
; ===============================================================
Isr_RC	;;	bcf	PIR1,RCIF	; geht nicht ! RCIF und TXIF sind nur zum Lesen
		call	SioRxD		; Byte in Ringpuffer laden
		goto	Isr_TestIF

; ===============================================================
; ISR Timer 2	Aufruf alle 0,1 ms
; ===============================================================
Isr_TMR2	bcf	PIR1,TMR2IF	; Int-Flag ruecksetzen
		bsf	STATUS,RP0	; Bank 1
		skip_1	PIE1,TMR2IE	; ... Timer 2-Interrupt ist enabled
		goto	Isr_TestIF	; ... ist nicht enabled

		bcf	STATUS,RP0	; Bank 0

		; ---------------------------
		; Aufruf alle 0,1 ms
		; ---------------------------
		; Ueberstrom testen
		; ---------------------------
TstImax	;;	skip_0	F_Imax		; ... noch kein Ueberstrom gemeldet
	;;	goto	TstImax_1

	;;	skip_1	iImax		; ... Ueberstrom liegt an
	;;	goto	TstImax_1

	;;	incf	zImax		; Zaehler + 1
	;;	movfw	pImax		; Ueberstrom-Zeit erreicht ?
	;;	subwf	zImax,W
	;;	skip_Z			; ... ja
	;;	goto	TstImax_2	; ... nein

	;;	bsf	F_Imax		; Flag Ueberstrom setzen
	;;	movlw	10		; 10 * 100ms
	;;	movwf	tImax		; Timer starten

TstImax_1 ;;	clrf	zImax		; Zaehler loeschen
TstImax_2

		; ---------------------------
		; Zaehler Duty
		; ---------------------------
TstDuty		bcf	F_iBit		; Bit erst mal aus
		movfw	zDuty		; Zaehler = 0 ?
		skip_NZ			; ... nein
		goto	TstDuty_1	; ... ja

		bsf	F_iBit		; Bit ein
		decf	zDuty		; Zaehler - 1

TstDuty_1	skip_1	F_iBit
		bcf	oPwm		; Ausgang Aus

		skip_0	F_iBit
		bsf	oPwm		; Ausgang Ein

		; ---------------------------
		; Zaehler Frequenz
		; ---------------------------
TstFreq		movfw	zFreq		; Zaehler = 0 ?
		skip_NZ			; ... nein
		goto	TstFrequ_1	; ... ja

		decf	zFreq		; Zaehler - 1
		goto	Tim01ms_1

TstFrequ_1	skip_0	F_Imax		; ... kein Ueberstrom
		goto	Tim01ms_1

		skip_1	F_Start		; ... Start Ein
		goto	Tim01ms_1

		skip_1	F_Takt		; ... Takt-Betrieb
		goto	TstFrequ_2

		skip_1	F_Tein		; ... Takt Ein
		goto	Tim01ms_1

TstFrequ_2	movfw	iFreq		; Zaehler Frequenz neu laden
		movwf	zFreq

		; ---------------------------
		; Zaehler Duty evtl. neu laden
		; ---------------------------
		movfw	iDuty		; Zaehler Duty neu laden
		movwf	zDuty

		; ---------------------------
		; Zaehler 0,1ms
		; ---------------------------
Tim01ms_1	decfsz	z01ms		; ... 0,1 ms-Zaehler - 1 = 0
		goto	Isr_TestIF	; ... sonst Ende

		; ---------------------------
		; Aufruf alle 1 ms
		; ---------------------------
		M_movlf	10,z01ms	; 0,1 ms-Zaehler = 10 (= 1,0 ms)
		skip_0	iImax		; ... kein Ueberstrom
		incf	zImax

		call	Rout10ms	; 10 Routinen je 10ms
		call	SioTxD		; Evtl. 1 Byte seriell ausgeben
		call	RdAdw		; A/D-Wandler einlesen
		decfsz	z1ms,F		; ... 1 ms-Zaehler - 1 = 0
		goto	Isr_TestIF	; ... sonst Ende

		; ---------------------------
		; Aufruf alle 10 ms
		; ---------------------------
		M_movlf	10,z1ms		; 1 ms-Zaehler = 10 (= 10 ms)

	; ....	Weitere Routinen bei "Rout10ms" einfuegen

		decfsz	z10ms,F		; ... 10 ms-Zaehler - 1 = 0
		goto	Isr_TestIF	; ... sonst Ende

		; ---------------------------
		; Aufruf alle 100 ms
		; ---------------------------
		M_movlf	10,z10ms	; 10 ms-Zaehler = 10 (= 100 ms)
		movfw	zImax		; Zaehler Ueberstrom = 0 ?
		skip_NZ			; ... nein
		goto	Tim100ms_2

		subwf	pImax,W		; Ueberstrom > Max ? (F - W = 80 - 81...255 = C => NC)
		skip_NC			; ... ja
		goto	Tim100ms_1

		bsf	F_Imax		; Flag Ueberstrom setzen
		movlw	10		; 10 * 100ms
		movwf	tImax		; Timer starten

Tim100ms_1	clrf	zImax		; Zaehler loeschen

Tim100ms_2	movfw	t100ms		; 100ms Timer = 0 ?
		skip_NZ			; ... nein
		goto	Tim100ms_3

		decf	t100ms		; 100ms Timer - 1; jetzt = 0 ?
		skip_NZ			; ... nein
		clrf	FlagsQ		; Alle Flags loeschen

Tim100ms_3	movfw	tImax		; Timer = 0 ?
		skip_NZ			; ... nein
		goto	Tim100ms_4

		decf	tImax		; Timer - 1; jetzt = 0 ?
		skip_NZ			; ... nein
		bcf	F_Imax		; Ueberstrom beendet

Tim100ms_4	incf	z100ms		; 100ms Zaehler + 1
		skip_0	z100ms,0	; ... Bit 0 = 0
		bsf	F_Bli1		; ... Blinker 100ms ein

		skip_1	z100ms,0	; ... Bit 0 = 1
		bcf	F_Bli1		; ... Blinker 100ms aus

		skip_0	z100ms,1	; ... Bit 1 = 0
		bsf	F_Bli2		; ... Blinker 200ms ein

		skip_1	z100ms,1	; ... Bit 1 = 1
		bcf	F_Bli2		; ... Blinker 200ms aus

		skip_0	z100ms,3	; ... Bit 3 = 0
		bsf	F_Bli8		; ... Blinker 800ms ein

		skip_1	z100ms,3	; ... Bit 3 = 1
		bcf	F_Bli8		; ... Blinker 800ms aus

	;;	movfw	TxDAnz		; Evtl. Anzahl verringern, falls nichts gesendet wird
	;;	skip_Z			; ... ist schon 0
	;;	decf	TxDAnz		; ... sonst - 1

		goto	Isr_TestIF

; ===============================================================
; EmpfangsByte im Ringpuffer ablegen
; ===============================================================
SioRxD		movfw	RCREG		; Empfangsbyte laden
		movwf	iTemp0		; und retten
		sublw	0x0D		; Return ?
		skip_NZ			; ... nein
		goto	SioRxD_1	; ... ja: Return speichern

		movfw	iTemp0
		sublw	0x0FF		; = 0x0FF ?
		skip_NZ			; ... nein
		return			; ... ja: verwerfen

		movfw	iTemp0
		sublw	0x1F		; Byte > 0x1F ?
		skip_NC			; ... ja
		return			; ... nein: verwerfen

SioRxD_1	movlw	LOW(RxDPuf)	; Puffer-Adresse
		addwf	RxDOfs,W	; plus Offset
		movwf	FSR		; fuer indirekte Adressierung
		bsf	STATUS,IRP	; Bank 2 / 3
		movfw	iTemp0		; EmpfangsByte laden
		movwf	INDF		; und im Puffer ablegen
		incf	RxDOfs		; Offset + 1
		movfw	RxDOfs
		sublw	RxDLen-1	; RxDOfs > RxDLen-1 ?
		skip_C			; ... nein
		clrf	RxDOfs		; ... ja: Offset = 0

		return

; ===============================================================
; Byte evtl. aus TxD-Puffer zum Senden in Transmit-Buffer ablegen
; Aufruf jede ms im Interrupt
; ===============================================================
SioTxD		movfw	TxDPos		; Byte zum Senden ?
		subwf	TxDOfs,W
		skip_NZ			; ... ja
		return			; ... nix zum Senden

		skip_1	PIR1,TXIF	; ... sendebereit
		return			; ... nicht sendebereit

		movlw	LOW(TxDPuf)	; Puffer-Adresse
		addwf	TxDOfs,W	; plus Offset
		movwf	FSR		; fuer indirekte Adressierung
		bsf	STATUS,IRP	; Bank 2 / 3
		movfw	INDF		; SendeByte aus Ringpuffer laden
		movwf	TXREG		; Byte senden (TXIF=0)
		incf	TxDOfs		; Offset + 1
		movfw	TxDOfs
		sublw	TxDLen-1	; TxDOfs > TxDLen-1 ?
		skip_C			; ... nein
		clrf	TxDOfs		; ... ja: Offset = 0

		movfw	TxDAnz		; 1 Byte weniger im TxD-Puffer
		skip_Z			; ... ist schon 0
		decf	TxDAnz		; ... sonst - 1

		return

; ===============================================================
; A/D-Wandler einlesen und naechsten Kanal setzen
; A/D-Wandler starten
; ===============================================================
RdAdw		movlw	HIGH(TblAdw)
		movwf	PCLATH
		movfw	z1ms
		andlw	00001111b
		goto	TblAdw

		; 1+6 -----------------------
RdAdw_1		movfw	ADRESH		; AD-Wert laden
		addwf	aAdw_5		; Alter plus neuer Wert
		rrf	aAdw_5		; / 2
		movlw	10001101b	; AD-Quelle auf Kanal AN1
		;	  001		= 2. Kanal (AN1)
		;       10		; FOSC/32
		;	     1		; GO
		;	       1	; ADON
		movwf	ADCON0		; AD-Quelle setzen
		return

		; 2+7 -----------------------
RdAdw_2		movfw	ADRESH		; AD-Wert laden
		addwf	aAdw_1		; Alter plus neuer Wert
		rrf	aAdw_1		; / 2
		movlw	10010101b	; AD-Quelle auf Kanal AN2
		;	  010		= 3. Kanal (AN2)
		movwf	ADCON0		; AD-Quelle setzen
		return

		; 3+8 -----------------------
RdAdw_3		movfw	ADRESH		; AD-Wert laden
		addwf	aAdw_2		; Alter plus neuer Wert
		rrf	aAdw_2		; / 2
		movlw	10011101b	; AD-Quelle auf Kanal AN3
		;	  011		= 4. Kanal (AN3)
		movwf	ADCON0		; AD-Quelle setzen
		return

		; 4+9 -----------------------
RdAdw_4		movfw	ADRESH		; AD-Wert laden
		addwf	aAdw_3		; Alter plus neuer Wert
		rrf	aAdw_3		; / 2
		movlw	10100101b	; AD-Quelle auf Kanal AN4
		;	  100		= 5. Kanal (AN4)
		movwf	ADCON0		; AD-Quelle setzen
		return

		; 5+10 ----------------------
RdAdw_5		movfw	ADRESH		; AD-Wert laden
		addwf	aAdw_4		; Alter plus neuer Wert
		rrf	aAdw_4		; / 2
		movlw	10000101b	; AD-Quelle auf Kanal AN0
		;	  000		= 1. Kanal (AN0)
		movwf	ADCON0		; AD-Quelle setzen
		return

; ===============================================================
; Diverse Routinen im Interrupt				10ms
; ===============================================================
Rout10ms	movlw	HIGH(TblRout10ms)
		movwf	PCLATH
		movfw	z1ms
		andlw	00001111b
		goto	TblRout10ms

; 1 -------------------------
; Taster 1
; ---------------------------
TstTast1	skip_1	iTast1		; ... Taster nicht betaetigt
		goto	TstTast1_5

		movfw	zTast1		; Zaehler = 0 ?
		skip_NZ			; ... nein
		return

		sublw	kPtime		; Zaehler > Prellzeit ?
		skip_NC			; ... ja
		goto	TstTast1_1	; L - W = 9 - 10...255 = C => NC

		movlw	kTlang + 1	; > 2,00 s ?
		subwf	zTast1,W	; F - W = 201...255 - 201 = NC => C
		skip_C			; ... ja
		bsf	F_Ta1K		; ... nein: Flag "Kurz" setzen

TstTast1_1	clrf	zTast1		; Zaehler loeschen
		return

		; ---------------------------
		; Taster betaetigt
		; ---------------------------
TstTast1_5	incf	zTast1,W	; Zaehler + 1 = Uebertrag ?
		skip_NZ			; ... nein
		return

		incf	zTast1		; Zaehler + 1
		movlw	kTlang		; 2,00 s erreicht ?
		subwf	zTast1,W	; F - W
		skip_NZ			; ... nein
		bsf	F_Ta1L		; ... ja: Flag "Lang" setzen

		return

; 2 -------------------------
; Taster 2 (mit Sonderfunktion)
; ---------------------------
TstTast2	skip_1	iTast2		; ... Taster nicht betaetigt
		goto	TstTast2_5

		; ---------------------------
		; Taster nicht betaetigt
		; ---------------------------
		movfw	zTast2		; Zaehler = 0 ?
		skip_NZ			; ... nein
		return

		sublw	kPtime		; Zaehler > Prellzeit ?
		skip_NC			; ... ja
		goto	TstTast2_1	; ... nein

		movlw	kTlang + 1	; > 2,00 s ?
		subwf	zTast2,W	; F - W = 201...255 - 201 = NC => C
		skip_C			; ... ja
		bsf	F_Ta2K		; ... nein: Flag "Kurz" setzen

TstTast2_1	clrf	zTast2		; Zaehler loeschen
		return

		; ---------------------------
		; Taster betaetigt
		; ---------------------------
TstTast2_5	skip_1	iTast1		; ... Taster 1 nicht betaetigt
		goto	TstTast2_9

		skip_1	iTast3		; ... Taster 3 nicht betaetigt
		goto	TstTast2_9

		incf	zTast2,W	; Zaehler + 1 = Uebertrag ?
		skip_NZ			; ... nein
		return

		incf	zTast2		; Zaehler + 1
		movlw	kTlang		; 2,00 s erreicht ?
		subwf	zTast2,W	; F - W
		skip_NZ			; ... nein
		bsf	F_Ta2L		; ... ja: Flag "Lang" setzen

		return

		; ---------------------------
		; Taste 2 + andere Taste
		; ---------------------------
TstTast2_9	bsf	F_Ta2X		; ... ja: Flag "X" setzen
		movlw	255
		movwf	zTast2
		return

; 3 -------------------------
; Taster 3
; ---------------------------
TstTast3	skip_1	iTast3		; ... Taster nicht betaetigt
		goto	TstTast3_5

		; ---------------------------
		; Taster nicht betaetigt
		; ---------------------------
		movfw	zTast3		; Zaehler = 0 ?
		skip_NZ			; ... nein
		return

		sublw	kPtime		; Zaehler > Prellzeit ?
		skip_NC			; ... ja
		goto	TstTast3_1	; ... nein

		movlw	kTlang + 1	; > 2,00 s ?
		subwf	zTast3,W	; F - W = 201...255 - 201 = NC => C
		skip_C			; ... ja
		bsf	F_Ta3K		; ... nein: Flag "Kurz" setzen

TstTast3_1	clrf	zTast3		; Zaehler loeschen
		return

		; ---------------------------
		; Taster betaetigt
		; ---------------------------
TstTast3_5	incf	zTast3,W	; Zaehler + 1 = Uebertrag ?
		skip_NZ			; ... nein
		return

		incf	zTast3		; Zaehler + 1
		movlw	kTlang		; 2,00 s erreicht ?
		subwf	zTast3,W	; F - W
		skip_NZ			; ... nein
		bsf	F_Ta3L		; ... ja: Flag "Lang" setzen

		return

; 4 -------------------------
; Fuss-Kontakt
; ---------------------------
TstFussK	skip_1	iFuKo		; ... Fuss-Kontakt nicht betaetigt
		goto	TstFussK_5

		; ---------------------------
		; Fuss-Kontakt nicht betaetigt
		; ---------------------------
		movfw	zFussK		; Zaehler = 0 ?
		skip_NZ			; ... nein
		return

		sublw	kPtime		; Zaehler > Prellzeit ?
		skip_NC			; ... ja
		goto	TstFussK_1	; ... nein

		movlw	50+1		; > 0,50 s ?
		subwf	zFussK,W	; F - W = 251...255 - 251 = NC => C
		skip_C			; ... ja
		bsf	F_FuKoK		; ... nein: Flag "Kurz" setzen

TstFussK_1	clrf	zFussK		; Zaehler loeschen
		return

		; ---------------------------
		; Fuss-Kontakt betaetigt
		; ---------------------------
TstFussK_5	incf	zFussK,W	; Zaehler + 1 = Uebertrag ?
		skip_NZ			; ... nein
		return

		incf	zFussK		; Zaehler + 1
		movlw	50		; 0,50 s erreicht ?
		subwf	zFussK,W	; F - W
		skip_NZ			; ... nein
		bsf	F_FuKoL		; ... ja: Flag "Lang" setzen

		return

; 5 -------------------------
; Ein-/Aus-Zeit bei Taktbetrieb testen
; ---------------------------
TstTakt		movfw	tTakt		; Timer = 0 ?
		skip_NZ			; ... nein
		goto	TstTakt_1

		decf	tTakt
		return

TstTakt_1	skip_1	F_Takt		; ... Takt-Betrieb
		return

		skip_1	F_Tein		; ... Ein-Zeit aktiv
		goto	SetTaktEin	; ... Aus-Zeit aktiv

SetTaktAus	bcf	F_Tein		; Aus-Zeit
		movlw	10		; Ruhe-Zeit + 10
		addwf	sRuhe,W
		skip_NC			; ... kein Uebertrag
		movlw	255

		movwf	tTakt		; Timer laden
		return

SetTaktEin	bsf	F_Tein		; Ein-Zeit
		movlw	10		; Ein-Zeit + 10
		addwf	sTakt,W
		skip_NC			; ... kein Uebertrag
		movfw	255

		movwf	tTakt		; Timer laden
		return

; 6 -------------------------
; Diverses
; ---------------------------
DecT10ms	bcf	oDigPot		; CS fuer Digital-Poti = aktiv
		movfw	iSpan		; Ist-Wert von Poti 1
		movwf	SSPBUF		; in SSPBUF schreiben

		movfw	t10ms		; 10 ms-Timer = 0 ?
		skip_Z			; ... ja
		decf	t10ms		; ... sonst - 1

		; Start Update Soll/Ist
		skip_0	F_UpdSI		; ... kann neu gesetzt werden
		return

		bsf	F_UpdSI		; Update Soll/Ist
		incf	zUpdIS		; Zaehler + 1
		return

; 7 -------------------------
; Led 1...7
; ---------------------------
SetLed7		skip_1	F_Takt		; ... Taktbetrieb
		goto	SetLed7_1

		skip_0	F_Bli1		; ... Blinker ist aus
		goto	SetLed7_1

SetLed7_0	movlw	11100000b	; Adr 3 = Leds aus
		movwf	iTemp0
		goto	SetLed7_5

SetLed7_1	skip_0	F_Start		; ... Start ist aus
		goto	SetLed7_2

		skip_1	F_Bli8		; ... Blinker ist ein
		goto	SetLed7_0

SetLed7_2	swapf	aWahl,W		; Bit 2...0 => 6...4
		andlw	01110000b
		movwf	iTemp0
		bcf	_C		; Carry loeschen
		rlf	iTemp0		; Bit 6...4 => 7...5

SetLed7_5	movfw	PORTB
		andlw	00011111b	; Bit 7...5 loeschen
		iorwf	iTemp0,W	; Led-Adr setzen
		movwf	PORTB
		return

; 8 -------------------------
; Led 8 und 9
; ---------------------------
SetLed8		skip_1	F_Imax		; ... Ueberstrom
		goto	SetLed8_0	; ... kein Ueberstrom

		skip_0	F_Bli1		; ... Blinker ist aus
		bcf	oLed8		; Led 8 ein

		skip_1	F_Bli1		; ... Blinker ist ein
SetLed8_0	bsf	oLed8		; Led 8 aus

SetLed9		skip_1	F_QStor		; ... Quittierung Store
		goto	SetLed9_1

		skip_1	F_Bli1		; ... Blinker ist ein
		bcf	oLed9		; Led 9 aus

		skip_0	F_Bli1		; ... Blinker ist aus
SetLed9_1	bsf	oLed9		; Led 9 ein

		return

; 10 ------------------------
; SPI-Ende fuer Digital-Poti
; ---------------------------
SpiTxD
		M_Bank1			; Bank 1
		skip_1	SSPSTAT,BF	; ... bertragung erfolgreich
		goto	SpiTxD_1	; ... Puffer noch nicht voll

		M_Bank0			; Bank 0
		movfw	SSPBUF		; Puffer auslesen
		movwf	iTemp0		; und retten
		bsf	oDigPot		; CS fuer Digital-Poti = inaktiv
		return

SpiTxD_1	M_Bank0			; Bank 0
		return

	include	"Tattoo_Init.asm"

; ***************************************************************
; Hauptprogramm-Schleife
; ***************************************************************
Main		call	UpdWahl		; Auswahl aktualisieren
		bcf	F_OK		; Bit erst mal 0
		movfw	aWahl		; Akt Wahl = 0 ?
		skip_Z			; ... ja
		bsf	F_OK

Main_1
		skip_1	F_UpdSI		; ... Soll-/Ist-Werte aktualisieren
		goto	Main_2

		call	UpdSollIst
		bcf	F_UpdSI		; Flag loeschen

Main_2

		; ---------------------------
		; Kommando auswerten
		; ---------------------------
Main_4		movfw	RxDPos		; CmdByte empfangen ?
		subwf	RxDOfs,W
		skip_NZ			; ... ja
		goto	Main_5_ret	; ... nein

		; ---------------------------
		; ACHTUNG !!!     Aufruf mit "goto Cmd_Ausw" und
		; Ruecksprung mit "goto Main_5_ret" wegen StackTiefe
		; ---------------------------
		M_Page1			; Rom-Page 1
		goto	Cmd_Ausw	; RxD-Byte speichern, evtl. Cmd-Puffer auswerten

Main_5_ret	goto	Main		; Hauptprogrammschleife


; ===============================================================
; Auswahl aktualisieren
; ===============================================================
UpdWahl		bcf	F_OK		; Bit = 0
		skip_0	F_Start		; Start ist 0
		bsf	F_OK		; Bit = 1
	
; -------------------------------------------
; Taste 1 auswerten
; -------------------------------------------
UpdWahl_1	skip_0	F_Ta1L		; ... Taste 1 lang nicht betaetigt
		goto	UpdWahl_15

		skip_1	F_Ta1K		; ... Taste 1 kurz betaetigt
		goto	UpdWahl_2

		skip_1	F_Ta2X		; ... Taste 2 ebenfalls betaetigt
		goto	UpdWahl_10

		; ---------------------------
		; Taste 1 + 2 kurz betaetigt
		; ---------------------------
		bcf	F_Ta1K		; Flag loeschen
		bcf	F_Ta2X

		return

		; ---------------------------
		; Taste 1 kurz betaetigt
		; ---------------------------
UpdWahl_10	bcf	F_Ta1K		; Flag loeschen
		movfw	aWahl		; Akt Wahl = 0 ?
		skip_Z			; ... ja
		goto	UpdWahl_11	; ... nein

		movlw	6
		movwf	aWahl		; Akt Wahl = 6
		return

UpdWahl_11	decf	aWahl		; Akt Wahl - 1
		return

		; ---------------------------
		; Taste 1 lang betaetigt
		; ---------------------------
UpdWahl_15	bcf	F_Ta1L		; Flag loeschen
		clrf	aWahl		; Akt Wahl = 0
		bcf	F_Takt		; Takt-Betrieb aus
		bcf	F_Start		; Stop
		return

; -------------------------------------------
; Taste 2 auswerten
; -------------------------------------------
UpdWahl_2	skip_0	F_Ta2L		; ... Taste 2 lang nicht betaetigt
		goto	UpdWahl_25

		skip_1	F_Ta2K		; ... Taste 2 kurz betaetigt
		goto	UpdWahl_3

		; ---------------------------
		; Taste 2 kurz betaetigt
		; Start / Stop
		; ---------------------------
		bcf	F_Ta2K		; Flag loeschen
		bcf	F_Ta2X
		skip_1	F_Start		; ... Start ist 1
		goto	UpdWahl_21

		bcf	F_Start		; Stop
		clrf	z100ms
		return

UpdWahl_21	bsf	F_Start		; Start
		return

		; ---------------------------
		; Taste 2 lang betaetigt
		; Store
		; ---------------------------
UpdWahl_25	bcf	F_Ta2L		; Flag loeschen
		movlw	HIGH(TblTast2)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblTast2

		; ---------------------------
Tast2_0		return

		; ---------------------------
Tast2_15	bsf	F_QStor		; Flag setzen
		movlw	20		; 20 * 100ms blinken
		movwf	t100ms
		goto	StorePara

		; ---------------------------
Tast2_6		return

; -------------------------------------------
; Taste 3 auswerten
; -------------------------------------------
UpdWahl_3	skip_0	F_Ta3L		; ... Taste 3 lang nicht betaetigt
		goto	UpdWahl_35

		skip_1	F_Ta3K		; ... Taste 3 kurz betaetigt
		goto	UpdWahl_4

		skip_1	F_Ta2X		; ... Taste 2 ebenfalls betaetigt
		goto	UpdWahl_30

		; ---------------------------
		; Taste 3 + 2 kurz betaetigt
		; ---------------------------
		bcf	F_Ta3K		; Flag loeschen
		bcf	F_Ta2X

		return

		; ---------------------------
		; Taste 3 kurz betaetigt
		; ---------------------------
UpdWahl_30	bcf	F_Ta3K		; Flag loeschen
		movfw	aWahl		; Akt Wahl > 5 ?
		sublw	6 - 1		; L - W = 5 - 6...255 = C => NC
		skip_NC			; ... ja
		goto	UpdWahl_31

		clrf	aWahl		; Akt Wahl = 0
		return

UpdWahl_31	incf	aWahl		; Akt Wahl + 1
		return

		; ---------------------------
		; Taste 3 lang betaetigt
		; ---------------------------
UpdWahl_35	bcf	F_Ta3L		; Flag loeschen
		skip_1	F_Takt		; ... Takt-Betrieb ist ein
		goto	UpdWahl_36

		bcf	F_Takt		; Normal-Betrieb
		return

UpdWahl_36	bsf	F_Takt		; Takt-Betrieb
		goto	SetTaktAus

; -------------------------------------------
; Fuss-Kontakt auswerten
; -------------------------------------------
UpdWahl_4	skip_0	F_FuKoL		; ... Fuss-Kontakt lang nicht betaetigt
		goto	UpdWahl_45

		skip_1	F_FuKoK		; ... Fuss-Kontakt kurz betaetigt
		return

		; ---------------------------
		; Fuss-Kontakt kurz betaetigt
		; ---------------------------
		bcf	F_FuKoK		; Flag loeschen
		skip_1	pModFk,0	; ... Modus 1 oder 3
		return			; ... Modus 0 oder 2

		skip_1	F_Start		; ... Start ist 1
		goto	UpdWahl_41

		bcf	F_Start		; Stop
		clrf	z100ms
		return

UpdWahl_41	bsf	F_Start		; Start
		return

		; ---------------------------
		; Fuss-Kontakt lang betaetigt
		; ---------------------------
UpdWahl_45	skip_1	pModFk,1	; ... Modus 2 oder 3
		return			; ... Modus 0 oder 1

		skip_0	F_FuKoL		; ... Fuss-Kontakt ist aus
		goto	UpdWahl_46

		bcf	F_Start		; Stop
		clrf	z100ms
		return

UpdWahl_46	bsf	F_Start		; Start
		return

; ===============================================================
; Akt Parameter im Ram und Eeprom speichern
; ===============================================================
StorePara	movlw	LOW(mSpan)	; Quell-Adr Man.-Werte
		movwf	Para1		; und retten
		decf	aWahl,W		; Akt Wahl - 1
		addlw	LOW(pSpanSp)	; + Ziel-Adr
		movwf	Para2		; und retten
		decf	aWahl,W		; Akt Wahl - 1
		addlw	LOW(eepSpanSp)	; + Eeprom-Adr
		movwf	Temp2		; und retten
		movlw	6		; 6 Werte
		movwf	zUni

StorePara_1	movfw	Para1		; Quell-Adresse zurueck
		movwf	FSR		; indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		movwf	Para0		; und retten

		movfw	Para2		; Ziel-Adresse zurueck
		movwf	FSR		; indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para0		; Wert zurueck
		movwf	INDF		; und im Ram speichern

		movfw	Temp2		; Eeprom-Adr zurueck
		movwf	AddrL
		movfw	Para0		; Wert zurueck
		M_Page1			; Rom-Page 1
		call	p1WrEeprom	; und im Eeprom speichern
		M_Page0			; Rom-Page 0

		incf	Para1		; Quell-Adr + 1
		movlw	5
		addwf	Para2		; Ziel-Adr + 5
		addwf	Temp2		; Eeprom-Adr + 5
		decfsz	zUni		; ... fertig
		goto	StorePara_1

		return

; ===============================================================
; Soll- und Ist-Werte aktualisieren
; ===============================================================
UpdSollIst	movlw	HIGH(TblUpd)
		movwf	PCLATH
		movfw	zUpdIS
		andlw	00000111b	; Nur 0...7
		goto	TblUpd

; -------------------------------------------
UpdSSpan	movfw	pVoltR		; Volt immer vom Regler ?
		skip_NZ			; ... nein, aus Speicher
		goto	UpdSSpan_0	; ... ja, vom Regler

		movlw	HIGH(TblUpdSSpan)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblUpdSSpan

		; ---------------------------
UpdSSpan_0	movfw	aAdw_1		; Wert laden
		movwf	Temp0
		bsf	STATUS,RP0	; Bank 1
		movfw	pSpanVon	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp1
		bsf	STATUS,RP0	; Bank 1
		movfw	pSpanBis	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp2
		call	CalcPoti	; Poti-Wert berechnen
		movwf	Temp0		; Wert retten
		movwf	mSpan		; und speichern

		; ---------------------------
		; Umwandeln von Poti-Wert nach Digi-Poti-Wert
		; (Poti-Wert - 50) * (256 / 160) .... [* 1,6]
		; ---------------------------
UpdSSpan_0_1	movlw	50		; Poti-Wert - 50
		subwf	Temp0,W		; F - W = 51...255 - 50 = NC => C
		skip_C			; ... kein Unterlauf
		movlw	0		; ... sonst = 0

		movwf	MathArgB1	; * 256 [* 1,6]
		clrf	MathArgB0
		movlw	160		; / 160
		movwf	MathArgA0
		clrf	MathArgA1
		call	p0Divu1616	; ArgB(16 Bit) / ArgA(16 Bit) -> ArgB(16 Bit),Rest(16 Bit)
		movfw	MathArgB1	; Ergebnis > 255 ?
		skip_NZ			; ... ja
		goto	UpdSSpan_0_2	; ... nein

		movlw	255		; Ergebnis = 255
		movwf	MathArgB0
		nop

UpdSSpan_0_2	movfw	MathArgB0	; Ergebnis nach W
		movwf	sSpan		; und speichern
		return

		; ---------------------------
UpdSSpan_15	decf	aWahl,W		; Akt Wahl 1...5 => 0...4
		addlw	LOW(pSpanSp)	; + Speicher-Adresse
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert kopieren
		movwf	Temp0
		goto	UpdSSpan_0_1

		; ---------------------------
UpdSSpan_6	goto	UpdSSpan_0

; -------------------------------------------
UpdSFreq	movlw	HIGH(TblUpdSFreq)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblUpdSFreq

		; ---------------------------
UpdSFreq_0	skip_0	F_Takt		; ... Poti 2 = Frequenz
		return

		movfw	aAdw_2		; Wert laden
		movwf	Temp0
		bsf	STATUS,RP0	; Bank 1
		movfw	pFreqVon	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp1
		bsf	STATUS,RP0	; Bank 1
		movfw	pFreqBis	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp2
		call	CalcPoti	; Poti-Wert berechnen
		movwf	mFreq		; und speichern

		; ---------------------------
		; Umwandeln von Frequenz nach 1/10 ms
		; 10000 / Frequenz
		; ---------------------------
UpdSFreq_0_1	movwf	MathArgA0	; Frequenz
		clrf	MathArgA1
		movlw	LOW(10000)	; 10000
		movwf	MathArgB0
		movlw	HIGH(10000)
		movwf	MathArgB1
		call	p0Divu1616	; ArgB(16 Bit) / ArgA(16 Bit) -> ArgB(16 Bit),Rest(16 Bit)
		movfw	MathArgB1	; Ergebnis > 255 ?
		skip_NZ			; ... ja
		goto	UpdSFreq_0_2	; ... nein

		movlw	255		; Ergebnis = 255
		movwf	MathArgB0
		nop

UpdSFreq_0_2	movfw	MathArgB0	; Ergebnis nach W
		movwf	sFreq		; und speichern
		return

		; ---------------------------
UpdSFreq_15	decf	aWahl,W		; Akt Wahl 1...5 => 0...4
		addlw	LOW(pFreqSp)	; + Speicher-Adresse
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert kopieren
		goto	UpdSFreq_0_1

		; ---------------------------
UpdSFreq_6	movlw	100		; Dauerlauf 10,0ms => 100Hz
		movwf	sFreq
		return

; -------------------------------------------
UpdSDuty	movlw	HIGH(TblUpdSDuty)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblUpdSDuty

		; ---------------------------
UpdSDuty_0	skip_0	F_Takt		; ... Poti 3 = DutyCycle
		return

		movfw	aAdw_3		; Wert laden
		movwf	Temp0
		bsf	STATUS,RP0	; Bank 1
		movfw	pDutyVon	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp1
		bsf	STATUS,RP0	; Bank 1
		movfw	pDutyBis	; Wert laden
		bcf	STATUS,RP0	; Bank 0
		movwf	Temp2
		call	CalcPoti	; Poti-Wert berechnen
		movwf	mDuty		; und speichern

		; ---------------------------
		; Umwandeln von Frequenz nach 1/10 ms
		; DutyCycle(ms) = Duty(%) * Frequenz(ms) / 100
		; ---------------------------
UpdSDuty_0_1	movwf	MathArgA0	; Duty(%)
		movfw	sFreq		; * Frequenz(ms)
		movwf	MathArgB0
		call	p0Mulu0808	; ArgB(8 Bit) * ArgA(8 Bit) -> ArgB(16 Bit)
		movlw	100		; / 100
		movwf	MathArgA0
		clrf	MathArgA1
		call	p0Divu1616	; ArgB(16 Bit) / ArgA(16 Bit) -> ArgB(16 Bit),Rest(16 Bit)
		movfw	MathArgB0	; Ergebnis nach W
		movwf	sDuty		; und speichern
		return

		; ---------------------------
UpdSDuty_15	decf	aWahl,W		; Akt Wahl 1...5 => 0...4
		addlw	LOW(pDutySp)	; + Speicher-Adresse
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert nach W
		goto	UpdSDuty_0_1

		; ---------------------------
UpdSDuty_6	movlw	100		; Dauerlauf 100% DutyCycle
		movwf	sDuty
		return

; -------------------------------------------
UpdSTakt	movlw	HIGH(TblUpdSTakt)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblUpdSTakt

		; ---------------------------
UpdSTakt_0	skip_1	F_Takt		; ... Poti 2 = Takt Ein-Zeit
		return

		movfw	aAdw_2		; Poti-Wert laden
		movwf	sTakt		; und speichern
		movwf	mTakt
		return

		; ---------------------------
UpdSTakt_15	decf	aWahl,W		; Akt Wahl 1...5 => 0...4
		addlw	LOW(pTaktSp)	; + Speicher-Adresse
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert kopieren
		movwf	sTakt
		return

		; ---------------------------
UpdSTakt_6
		return

; -------------------------------------------
UpdSRuhe	movlw	HIGH(TblUpdSRuhe)
		movwf	PCLATH
		movfw	aWahl
		andlw	00000111b	; Nur 0...7
		goto	TblUpdSRuhe

		; ---------------------------
UpdSRuhe_0	skip_1	F_Takt		; ... Poti 3 = Takt Aus-Zeit
		return

		movfw	aAdw_3		; Poti-Wert laden
		movwf	sRuhe		; und speichern
		movwf	mRuhe
		return

		; ---------------------------
UpdSRuhe_15	decf	aWahl,W		; Akt Wahl 1...5 => 0...4
		addlw	LOW(pRuheSp)	; + Speicher-Adresse
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert kopieren
		movwf	sRuhe
		return

		; ---------------------------
UpdSRuhe_6
		return

; -------------------------------------------
UpdISpan	movfw	iSpan		; Ist-Wert laden
		movwf	Temp0		; und retten
		movfw	sSpan		; Soll-Wert laden
		movwf	Temp1		; und retten
		movfw	pGeschw
		movwf	Temp2
		call	UpdIstX		; Ist an Soll annaehern
		movwf	iSpan		; Neuen Ist-Wert speichern
		return

		; ---------------------------
UpdIFreq	movfw	iFreq		; Ist-Wert laden
		movwf	Temp0		; und retten
		movfw	sFreq		; Soll-Wert laden
		movwf	Temp1		; und retten
		movfw	pGeschw
		movwf	Temp2
		call	UpdIstX		; Ist an Soll annaehern
		movwf	iFreq		; Neuen Ist-Wert speichern
		return

		; ---------------------------
UpdIDuty	movfw	iDuty		; Ist-Wert laden
		movwf	Temp0		; und retten
		movfw	sDuty		; Soll-Wert laden
		movwf	Temp1		; und retten
		movfw	pGeschw
		movwf	Temp2
		call	UpdIstX		; Ist an Soll annaehern
		movwf	iDuty		; Neuen Ist-Wert speichern
		return

; ===============================================================
; Ist-Wert an Soll-Wert annaehern
; Eingang:	Temp0	Ist-Wert
;		Temp1	Soll-Wert
;		Temp2	Geschwindigkeit
; Ausgang:	W & Temp0	Neuer/alter Ist-Wert
; ===============================================================
UpdIstX		movfw	Temp2		; Geschwindigkeit = 0 ?
		skip_NZ			; ... nein
		goto	UpdIstX_3	; ... ja: Soll --> Ist

		; ---------------------------
		; Richtung ermitteln: Soll - Ist
		; ---------------------------
		movfw	Temp0		; Soll - Ist
		subwf	Temp1,W
		skip_NZ			; ... Soll <> Ist
		goto	UpdIstX_5	; ... Soll == Ist

		skip_C			; ... positiv: also rauf
		goto	UpdIstX_2	; ... negativ: runter

		; ---------------------------
		; Ist-Wert rauf
		; ---------------------------
		movfw	Temp2		; Ist + Steps
		addwf	Temp0
		skip_NC			; ... kein Uebertrag
		goto	UpdIstX_3	; ... Uebertrag: Soll --> Ist

		movfw	Temp0		; Ist <= Soll ? 
		subwf	Temp1,W		; F - W = Soll - Ist = 90 - 0...90 = NC => C
		skip_NC			; ... Soll <= Ist
		goto	UpdIstX_5	; ... Soll > Ist

		goto	UpdIstX_3	; Soll --> Ist

		; ---------------------------
		; Ist-Wert runter
		; ---------------------------
UpdIstX_2	movfw	Temp2		; Ist - Steps
		subwf	Temp0		; F - W = Ist - Step = 5...100 - 5 = NC => C
		skip_C			; ... kein Unterlauf
		goto	UpdIstX_3	; ... Unterlauf: Ist --> Soll

		movfw	Temp1		; Ist >= Soll ?
		subwf	Temp0,W		; F - W = Ist - Soll = 5...100 - 5 = NC => C
		skip_NC			; ... Ist < Soll
		goto	UpdIstX_5	; ... Ist >= Soll

UpdIstX_3	movfw	Temp1		; Soll --> Ist
		movwf	Temp0
		nop

		; ---------------------------
		; Neuen/alten Ist-Wert setzen
		; ---------------------------
UpdIstX_5	movfw	Temp0		; Neuen Ist-Wert nach W
		return

; ===============================================================
; Poti-Wert berechnen
; Eingang:	Temp0	AD-Wert
;		Temp1	Von-Wert
;		Temp2	Bis-Wert
; Ausgang:	W	Poti-Wert
;
; Poti-Wert = (Diff * AD-Wert / 256) + Von-Wert
; ===============================================================
CalcPoti	movfw	Temp1		; Diff = Bis - Von
		subwf	Temp2,W		; F - W = Bis - Von
		movwf	MathArgB0	; Differenz
		movfw	Temp0		; * AD-Wert
		movwf	MathArgA0
		call	p0Mulu0808	; ArgB(8 Bit) * ArgA(8 Bit) -> ArgB(16 Bit)
		movfw	MathArgB1	; / 256
		addwf	Temp1,W		; + Von
		return

; ***************************************************************
; DIVERSE ROUTINEN
; ***************************************************************
; ===============================================================
; Ram indirekt loeschen
; ===============================================================
ResRam		movlw	LOW(RamBeg0)	; Ram-Anfang
		movwf	FSR
		bcf	STATUS,IRP	; Bank 0 / 1
		movlw	RamAnz0		; Anzahl Adressen
		movwf	zUni
ResRam_1	clrf	INDF		; Ram indirekt loeschen
		incf	FSR
		decfsz	zUni		; ... Zaehler - 1 = 0
		goto	ResRam_1

		clrf	Flags0		; Flags loeschen
		clrf	Flags1
		return

; ===============================================================
; Text fuer Start senden
; ===============================================================
SendStart	movfw	Temp0
		sublw	2		; = 2 (WarmStart) ?
		skip_NZ			; ... nein
		goto	SendWStart	; ... ja: WarmStart

SendKStart	movlw	LOW(TxtSysErr_1)	; Kalt-Start
		movwf	AddrL
		movlw	HIGH(TxtSysErr_1)
		movwf	AddrH
		goto	SendXStart

SendWStart	movlw	LOW(TxtSysErr_2)	; Warm-Start
		movwf	AddrL
		movlw	HIGH(TxtSysErr_2)
		movwf	AddrH

SendXStart	M_Page1			; Rom-Page 1
		call	p1PrnCrLf	; cr/lf senden
		call	p1PrnStrF	; String senden
		call	p1PrnCrLf	; cr/lf senden
		M_Page0			; Rom-Page 0
		return

; ****************************************************************
; Multiplikation : ArgB(8 Bit) * ArgA(8 Bit) -> ArgB(16 Bit)
; ****************************************************************
p0Mulu0808	movlw	8		; Zaehler = 8
		movwf	MathZlr0
		movfw	MathArgB0	; ArgB -> W
		clrf	MathArgB1	; Ergebnis MSB loeschen
		bcf	_C		; Carry loeschen
p0Mulu0808_1	rrf	MathArgA0
		skip_NC			; ... Bit = 0
		addwf	MathArgB1	; ... Bit = 1

		rrf	MathArgB1	; C und Ergebnis rechts schieben
		rrf	MathArgB0
		decfsz	MathZlr0	; Fertig ?
		goto	p0Mulu0808_1	; ... nein

		return

; ***************************************************************
; Division : ArgB(16 Bit) / ArgA(16 Bit) -> ArgB(16 Bit),Rest(16 Bit)
; ***************************************************************
p0Divu1616	movlw	16		; Zaehler = 16
		movwf	MathZlr0
		movfw	MathArgB0	; ArgB nach Tmp
		movwf	MathTmp0
		movfw	MathArgB1
		movwf	MathTmp1
		clrf	MathArgB0	; Ergebnis loeschen
		clrf	MathArgB1
		clrf	MathErg0	; Rest loeschen
		clrf	MathErg1
p0Divu1616_1	bcf	_C		; Bits nach links schieben
		rlf	MathTmp0
		rlf	MathTmp1
		rlf	MathErg0
		rlf	MathErg1
		movfw	MathArgA1
		subwf	MathErg1,W	; ArgA > Rest ?
		btfss	_Z		; ... ist gleich
		goto	p0Divu1616_2	; ... ist ungleich

		movfw	MathArgA0
		subwf	MathErg0,W	; if msb equal then check lsb
p0Divu1616_2	skip_C			; Wenn Rest > ArgA, dann ist C = 1
		goto	p0Divu1616_3

		movfw	MathArgA0	; Rest - ArgA -> Rest
		subwf	MathErg0
		skip_C			; ... kein Unterlauf
 		decf	MathErg1	; High-Byte - 1

		movfw	MathArgA1
		subwf	MathErg1
		bsf	_C
p0Divu1616_3	rlf	MathArgB0	; Ergebnis links schieben
		rlf	MathArgB1
		decfsz	MathZlr0	; Fertig ?
		goto	p0Divu1616_1	; ... nein

		return

; ###############################################################
	org	0x0700

; ---------------------------------------------------------------
; Sprung zur ADW-Routine
; ---------------------------------------------------------------
TblAdw		addwf	PCL
		return			; 0 gibt es nicht
		return			; 1  = 10.
		goto	RdAdw_5		; 2
		return			; 3
		goto	RdAdw_4		; 4
		return			; 5
		goto	RdAdw_3		; 6
		return			; 7
		goto	RdAdw_2		; 8
		return			; 9
		goto	RdAdw_1		; 10 = 1.
		return			; 11 Rest gibt es nicht
		return			; 12
		return			; 13
		return			; 14
		return			; 15

; ---------------------------------------------------------------
; Sprung zur Update-Routine
; ---------------------------------------------------------------
TblUpd		addwf	PCL
		goto	UpdSSpan	; 0
		goto	UpdSFreq	; 1
		goto	UpdSDuty	; 2
		goto	UpdSTakt	; 3
		goto	UpdSRuhe	; 4
		goto	UpdISpan	; 5
		goto	UpdIFreq	; 6
		goto	UpdIDuty	; 7

TblUpdSSpan	addwf	PCL
		goto	UpdSSpan_0	; 0
		goto	UpdSSpan_15	; 1
		goto	UpdSSpan_15	; 2
		goto	UpdSSpan_15	; 3
		goto	UpdSSpan_15	; 4
		goto	UpdSSpan_15	; 5
		goto	UpdSSpan_6	; 6
		return			; 7

TblUpdSFreq	addwf	PCL
		goto	UpdSFreq_0	; 0
		goto	UpdSFreq_15	; 1
		goto	UpdSFreq_15	; 2
		goto	UpdSFreq_15	; 3
		goto	UpdSFreq_15	; 4
		goto	UpdSFreq_15	; 5
		goto	UpdSFreq_6	; 6
		return			; 7

TblUpdSDuty	addwf	PCL
		goto	UpdSDuty_0	; 0
		goto	UpdSDuty_15	; 1
		goto	UpdSDuty_15	; 2
		goto	UpdSDuty_15	; 3
		goto	UpdSDuty_15	; 4
		goto	UpdSDuty_15	; 5
		goto	UpdSDuty_6	; 6
		return			; 7

TblUpdSTakt	addwf	PCL
		goto	UpdSTakt_0	; 0
		goto	UpdSTakt_15	; 1
		goto	UpdSTakt_15	; 2
		goto	UpdSTakt_15	; 3
		goto	UpdSTakt_15	; 4
		goto	UpdSTakt_15	; 5
		goto	UpdSTakt_6	; 6
		return			; 7

TblUpdSRuhe	addwf	PCL
		goto	UpdSRuhe_0	; 0
		goto	UpdSRuhe_15	; 1
		goto	UpdSRuhe_15	; 2
		goto	UpdSRuhe_15	; 3
		goto	UpdSRuhe_15	; 4
		goto	UpdSRuhe_15	; 5
		goto	UpdSRuhe_6	; 6
		return			; 7

TblTast2	addwf	PCL
		goto	Tast2_0		; 0
		goto	Tast2_15	; 1
		goto	Tast2_15	; 2
		goto	Tast2_15	; 3
		goto	Tast2_15	; 4
		goto	Tast2_15	; 5
		goto	Tast2_6		; 6
		return			; 7

; ---------------------------------------------------------------
; Sprung zu diversen Routinen alle 10ms im Interrupt	1 ms
; ---------------------------------------------------------------
TblRout10ms	addwf	PCL
		return			; 0 gibt es nicht
		goto	SpiTxD		; 1  = 10.
		return			; 2
		goto	SetLed8		; 3
		goto	SetLed7		; 4
		goto	TstTakt		; 5
		goto	DecT10ms	; 6
		goto	TstFussK	; 7
		goto	TstTast3	; 8
		goto	TstTast2	; 9
		goto	TstTast1	; 10 = 1.
		return			; 11 Rest gibt es nicht
		return			; 12
		return			; 13
		return			; 14
		return			; 15

; ###############################################################
; Programmspeicher Page 1 (0x0800...0x0FFF)
; ###############################################################
	org	0x0800

p1Reset		nop		; 00h Resetvektor
		goto	P1Init	; 01h Resetvektor
		goto	P1Init	; 02h Resetvektor
		goto	P1Init	; 03h Resetvektor

P1Isr		M_Page0		; auf Rom-Page 0 schalten
		goto	Interrupt

P1Init		M_Page0		; auf Rom-Page 0 schalten
		goto	ProgBeg

; ***************************************************************
; Kommando auswerten
; ***************************************************************
; ===============================================================
; Cmd-Puffer ruecksetzen
; ===============================================================
p1ResCmdPuf	M_Bank2
		clrf	CmdPuf
		M_Bank0
		clrf	CmdPos
		return

; ===============================================================
; RxD-Byte in Cmd-Puffer ablegen
; Bei CR Cmd-Puffer auswerten
; ===============================================================
Cmd_Ausw	call	p1GetRxDByte	; Byte aus RxD-Puffer laden

		; ---------------------------
		; Byte auf CR testen
		; ---------------------------
		movfw	CmdByte		; Byte laden
		sublw	0x0D		; CR ?
		skip_NZ			; ... nein
		goto	Cmd_Ret

		; ---------------------------
		; Byte auf Doppelpunkt testen
		; ---------------------------
		movfw	CmdByte		; Byte laden
		sublw	":"		; Doppelpunkt ?
		skip_NZ			; ... nein
		goto	Cmd_Ret

		; ---------------------------
		; Byte auf Semikolon testen
		; ---------------------------
		movfw	CmdByte		; Byte laden
		sublw	";"		; Semikolon ?
		skip_NZ			; ... nein
		goto	Cmd_Semi

		; ---------------------------
		; Kommentar verwerfen
		; ---------------------------
		skip_0	F_Semi		; ... kein Kommentar
		goto	Cmd_Ausw_E	; ... sonst verwerfen

		goto	Cmd_Ausw_1	; Byte speichern

		; ---------------------------
		; Bei CR Flag setzen und 0 speichern
		; ---------------------------
Cmd_Ret		bsf	F_Ret		; Kommando ausfuehren
		bcf	F_Semi		; Alles ruecksetzen
		clrf	CmdByte		; Null anfuegen
		goto	Cmd_Ausw_1

		; ---------------------------
		; Bei Semikolon Flag setzen
		; ---------------------------
Cmd_Semi	bsf	F_Semi		; Ab jetzt Kommentar, alles verwerfen
		goto	Cmd_Ausw_E

		; ---------------------------
		; Byte im Cmd-Puffer ablegen
		; ---------------------------
Cmd_Ausw_1	movlw	LOW(CmdPuf)	; Cmd-Puffer-Adresse
		addwf	CmdPos,W	; plus Offset
		movwf	FSR		; fuer indirekte Adressierung
		bsf	STATUS,IRP	; Bank 2 / 3
		movfw	CmdByte		; Cmd-Byte laden
		movwf	INDF		; und im Cmd-Puffer ablegen
		movfw	CmdPos
		sublw	CmdLen-2	; CmdPos > CmdLen-2 ?
		skip_NC			; ... ja
		incf	CmdPos		; ... nein: Offset + 1

		; ---------------------------
		; Nach CR oder Doppelpunkt Kommando ausfuehren
		; ---------------------------
		skip_1	F_Ret		; ... es war CR / Doppelpunkt
		goto	Cmd_Ausw_E

Cmd_Ausw_X	bcf	F_Fmt		; Zahlen-Formatierung aus
		bcf	F_000		; Keine fuehrenden Nullen
		bcf	F_Ret		; Flags ruecksetzen
		bcf	F_Eep
		clrf	CmdPos		; Offset = 0
		call	p1SuchA_Z	; Cmd-Buchstabe suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_1	; ... nicht gefunden

		movlw	"?"		; Fragezeichen ?
		subwf	CmdByte,W
		skip_NZ			; ... nein
		goto	Cmd_Hilfe

	;;	movlw	"#"		; Numernzeichen ?
	;;	subwf	CmdByte,W
	;;	skip_NZ			; ... nein
	;;	goto	Cmd_Yinc

		movlw	HIGH(TblCmd)
		movwf	PCLATH
		movlw	"A"		; Tabellen-Offset berechnen
		subwf	CmdByte,W	; F - W --> W
		goto	TblCmd

; ---------------------------------------------------------------
Cmd_Fertig	call	p1ResCmdPuf
Cmd_Ausw_E	M_Page0
		goto	Main_5_ret

; ===============================================================
; FehlerMeldungen
; ===============================================================
PrnCmdErr_1	movlw	LOW(TxtCmdErr_1)	; Kommando fehlt
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_1)
		goto	PrnCmdErr

PrnCmdErr_2	movlw	LOW(TxtCmdErr_2)	; Kommando falsch
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_2)
		goto	PrnCmdErr

PrnCmdErr_3	movlw	LOW(TxtCmdErr_3)	; Parameter fehlt / falsch
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_3)
		goto	PrnCmdErr

PrnCmdErr_4	movlw	LOW(TxtCmdErr_4)	; Parameter zu klein
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_4)
		goto	PrnCmdErr

PrnCmdErr_5	movlw	LOW(TxtCmdErr_5)	; Parameter zu gross
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_5)
		goto	PrnCmdErr

PrnCmdErr_6	movlw	LOW(TxtCmdErr_6)	; Bestaetigung fehlt / falsch
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_6)
		goto	PrnCmdErr

PrnCmdErr_7	movlw	LOW(TxtCmdErr_7)	; Speicher-Nr. fehlt
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_7)
		goto	PrnCmdErr

PrnCmdErr_8	movlw	LOW(TxtCmdErr_8)	; Param 1 > Param 2
		movwf	AddrL
		movlw	HIGH(TxtCmdErr_8)
PrnCmdErr	movwf	AddrH
		call	p1PrnStrF		; String senden
		call	p1PrnCrLf		; cr/lf senden
		goto	Cmd_Fertig

; ===============================================================
; P - Programm-Version
; ===============================================================
Cmd_Prog	movlw	LOW(TxtCVers)
		movwf	AddrL
		movlw	HIGH(TxtCVers)
		movwf	AddrH
		call	p1PrnStrF
		goto	Cmd_Fertig

; ===============================================================
; ? - Hilfe
; ===============================================================
Cmd_Hilfe	movlw	LOW(TxtHilfe)
		movwf	AddrL
		movlw	HIGH(TxtHilfe)
		movwf	AddrH
		call	p1PrnStrF
		goto	Cmd_Fertig

; ===============================================================
; W - Werte anzeigen
; ===============================================================
Cmd_Werte	movlw	LOW(TxtWerte)
		movwf	AddrL
		movlw	HIGH(TxtWerte)
		movwf	AddrH
		call	p1PrnStrF	; String aus Flash-Speicher senden
		bsf	F_Fmt		; Formatierung ein

		bcf	F_000		; Fuehrende Nullen aus
		call	p1PrnStrF	; String aus Flash-Speicher senden
		call	p1PrnSpanPw
		call	p1PrnStrF
		call	p1PrnFreqPw
		call	p1PrnStrF
		call	p1PrnDutyPw

		call	p1PrnStrF
		call	p1Prn5XSpanSp
		call	p1PrnStrF
		call	p1Prn5YFreqSp
		call	p1PrnStrF
		call	p1Prn5ZDutySp
		call	p1PrnStrF
		call	p1Prn5TaktSp
		call	p1PrnStrF
		call	p1Prn5RuheSp

		call	p1PrnStrF
		call	p1PrnUStrom
		call	p1PrnStrF
		call	p1PrnGeschw
		call	p1PrnStrF
		call	p1PrnModFk
		call	p1PrnStrF
		call	p1PrnVoltR
		call	p1PrnStrF
		call	p1PrnAktWahl
		call	p1PrnStrF
		call	p1Prn5Ana
		call	p1PrnStrF
		call	p1Prn5Man
		call	p1PrnStrF
		call	p1Prn5Sol
		call	p1PrnStrF

		movlw	LOW(TxtCVersX)
		movwf	AddrL
		movlw	HIGH(TxtCVersX)
		movwf	AddrH
		call	p1PrnStrF
		goto	Cmd_Fertig

; ===============================================================
; Aktuelle Wahl				A 1...7
; ===============================================================
Cmd_AktWahl	call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_AktWahl_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; Wert retten
		sublw	7		; <= 7 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; Wert zurueck; = 0 ?
		skip_NZ			; ... nein
		goto	PrnCmdErr_4	; ... ja: Parameter zu klein

		decf	Para1
		nop

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_AktWahl_5	movfw	Para1		; Wert zurueck
		movwf	aWahl		; und speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_AktWahl_7	call	p1PrnAAktWahl
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "A" und Wert senden
		; ---------------------------
p1PrnAAktWahl	movlw	"A"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Wert senden
		; ---------------------------
p1PrnAktWahl	incf	aWahl,W		; Akt Wahl + 1
		goto	p1PrnZif1

; ===============================================================
; Geschw. Soll-Ist-Annaeherung		G 0...20
; ===============================================================
Cmd_Geschw	call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_Geschw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; Wert retten
		sublw	20		; <= 20 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_Geschw_5	movfw	Para1		; Wert zurueck
		movwf	pGeschw		; und speichern

		movlw	LOW(eepGeschw)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_Geschw_7	call	p1PrnXGeschw
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "G" und Wert senden
		; ---------------------------
p1PrnXGeschw	movlw	"G"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Wert senden
		; ---------------------------
p1PrnGeschw	movfw	pGeschw
		goto	p1PrnZif30

; ===============================================================
; Modus Fuss-Kontakt			M 0...15
; ===============================================================
Cmd_ModFk	call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_ModFk_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; Wert retten
		sublw	15		; <= 15 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_ModFk_5	movfw	Para1		; Wert zurueck
		movwf	pModFk		; und speichern

		movlw	LOW(eepModFk)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_ModFk_7	call	p1PrnXModFk
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "M" und Wert senden
		; ---------------------------
p1PrnXModFk	movlw	"M"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Wert senden
		; ---------------------------
p1PrnModFk	movfw	pModFk
		goto	p1PrnZif30

; ===============================================================
; UeberStrom-Zeit			U 10...90
; ===============================================================
Cmd_UStrom	call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_UStrom_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; Wert retten
		sublw	10-1		; > 9 ?
		skip_NC			; ... ja
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Wert zurueck
		sublw	90		; <= 90 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_UStrom_5	movfw	Para1		; Wert zurueck
		movwf	pImax		; und speichern

		movlw	LOW(eepImax)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_UStrom_7	call	p1PrnXUStrom
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "U" und Wert senden
		; ---------------------------
p1PrnXUStrom	movlw	"U"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Wert senden
		; ---------------------------
p1PrnUStrom	movfw	pImax
		goto	p1PrnZif30

; ===============================================================
; Volt immer vom Regler			V 0...1
; ===============================================================
Cmd_VoltR	call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_VoltR_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; Wert retten
		sublw	1		; <= 1 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_VoltR_5	movfw	Para1		; Wert zurueck
		movwf	pVoltR		; und speichern

		movlw	LOW(eepVoltR)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_VoltR_7	call	p1PrnXVoltR
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "V" und Wert senden
		; ---------------------------
p1PrnXVoltR	movlw	"V"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Wert senden
		; ---------------------------
p1PrnVoltR	movfw	pVoltR
		goto	p1PrnZif1

; ===============================================================
; Spannung PotiWerte Von/Bis		S 50...200 60...210
; ===============================================================
Cmd_SpanPw	call	p1Such0_9	; 1. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_SpanPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; 1. Wert retten
		call	p1Such0_9	; 2. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_SpanPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; 2. Wert retten
		sublw	60-1		; > 59 ?
		skip_NC			; ... ja
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; 2. Wert zurueck
		sublw	210		; <= 210 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		sublw	50-1		; > 49 ?	L - W = 49 -  0... 49 = NC => C
		skip_NC			; ... ja	L - W = 49 - 50...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; 1. Wert zurueck
		sublw	200		; <= 200 ?	L - W = 100 -   0...100 = NC => C
		skip_C			; ... ja	L - W = 100 - 101...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		addlw	10		; + 10
		subwf	Para2,W		; < 2. Wert ?	F - W = 2 - 1 = NC => C
		skip_C			; ... ja	F - W = 1 - 2 = C => NC
		goto	PrnCmdErr_8	; ... nein: Param 2 - Param 1 < 10

		; ---------------------------
		; Werte speichern
		; ---------------------------
Cmd_SpanPw_5	movfw	Para1		; 1. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pSpanVon	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepSpanVon)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; 1. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		movfw	Para2		; 2. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pSpanBis	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepSpanBis)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para2		; 2. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Werte senden
		; ---------------------------
Cmd_SpanPw_7	call	p1PrnXSpanPw
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "S" und Werte senden
		; ---------------------------
p1PrnXSpanPw	movlw	"S"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Werte senden
		; ---------------------------
p1PrnSpanPw	bsf	STATUS,RP0	; Bank 1
		movfw	pSpanVon
		bcf	STATUS,RP0	; Bank 0
		call	p1PrnZif31
		call	p1PrnSpc	; Leerzeichen senden
		bsf	STATUS,RP0	; Bank 1
		movfw	pSpanBis
		bcf	STATUS,RP0	; Bank 0
		goto	p1PrnZif31

; ===============================================================
; Frequenz PotiWerte Von/Bis		F 40...190 50...200
; ===============================================================
Cmd_FreqPw	call	p1Such0_9	; 1. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_FreqPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; 1. Wert retten
		call	p1Such0_9	; 2. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_FreqPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; 2. Wert retten
		sublw	50-1		; > 49 ?	L - W = 49 -  0... 49 = NC => C
		skip_NC			; ... ja	L - W = 49 - 50...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; 2. Wert zurueck
		sublw	200		; <= 200 ?	L - W = 200 -   0...200 = NC => C
		skip_C			; ... ja	L - W = 200 - 201...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		sublw	50-1		; > 49 ?
		skip_NC			; ... ja
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; 1. Wert zurueck
		sublw	100		; <= 100 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		addlw	10		; + 10
		subwf	Para2,W		; < 2. Wert ?	F - W = 2 - 1 = NC => C
		skip_C			; ... ja	F - W = 1 - 2 = C => NC
		goto	PrnCmdErr_8	; ... nein: Param 2 - Param 1 < 10

		; ---------------------------
		; Werte speichern
		; ---------------------------
Cmd_FreqPw_5	movfw	Para1		; 1. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pFreqVon	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepFreqVon)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; 1. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		movfw	Para2		; 2. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pFreqBis	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepFreqBis)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para2		; 2. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Werte senden
		; ---------------------------
Cmd_FreqPw_7	call	p1PrnXFreqPw
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "F" und Werte senden
		; ---------------------------
p1PrnXFreqPw	movlw	"F"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Werte senden
		; ---------------------------
p1PrnFreqPw	bsf	STATUS,RP0	; Bank 1
		movfw	pFreqVon
		bcf	STATUS,RP0	; Bank 0
		call	p1PrnZif30
		call	p1PrnSpc	; Leerzeichen senden
		bsf	STATUS,RP0	; Bank 1
		movfw	pFreqBis
		bcf	STATUS,RP0	; Bank 0
		goto	p1PrnZif30

; ===============================================================
; Ein-Zeit PotiWerte Von/Bis		D 10...80 20...90
; ===============================================================
Cmd_DutyPw	call	p1Such0_9	; 1. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_DutyPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para1		; 1. Wert retten
		call	p1Such0_9	; 2. Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_DutyPw_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; 2. Wert retten
		sublw	20-1		; > 19 ?	L - W = 19 -  0... 19 = NC => C
		skip_NC			; ... ja	L - W = 19 - 20...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; 2. Wert zurueck
		sublw	90		; <= 90 ?	L - W = 90 -  0... 90 = NC => C
		skip_C			; ... ja	L - W = 90 - 91...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		sublw	10-1		; > 9 ?
		skip_NC			; ... ja
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; 1. Wert zurueck
		sublw	80		; <= 80 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		movfw	Para1		; 1. Wert zurueck
		addlw	10		; + 10
		subwf	Para2,W		; < 2. Wert ?	F - W = 2 - 1 = NC => C
		skip_C			; ... ja	F - W = 1 - 2 = C => NC
		goto	PrnCmdErr_8	; ... nein: Param 2 - Param 1 < 10

		; ---------------------------
		; Werte speichern
		; ---------------------------
Cmd_DutyPw_5	movfw	Para1		; 1. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pDutyVon	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepDutyVon)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para1		; 1. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		movfw	Para2		; 2. Wert zurueck
		bsf	STATUS,RP0	; Bank 1
		movwf	pDutyBis	; und speichern
		bcf	STATUS,RP0	; Bank 0

		movlw	LOW(eepDutyBis)	; Eeprom-Adresse
		movwf	AddrL
		movfw	Para2		; 2. Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Werte senden
		; ---------------------------
Cmd_DutyPw_7	call	p1PrnXDutyPw
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "D" und Werte senden
		; ---------------------------
p1PrnXDutyPw	movlw	"D"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; Werte senden
		; ---------------------------
p1PrnDutyPw	bsf	STATUS,RP0	; Bank 1
		movfw	pDutyVon
		bcf	STATUS,RP0	; Bank 0
		call	p1PrnZif30
		call	p1PrnSpc	; Leerzeichen senden
		bsf	STATUS,RP0	; Bank 1
		movfw	pDutyBis
		bcf	STATUS,RP0	; Bank 0
		goto	p1PrnZif30

; ===============================================================
; Spannung Speicher 1...5		X 1...5 50...210
; ===============================================================
Cmd_XSpanSp	call	p1Such0_9	; Nr suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_7	; ... nicht gefunden: SpeicherNr fehlt

		movwf	Para1		; Nr retten
		call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_XSpanSp_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; Wert retten
		movfw	Para1		; Nr zurueck
		skip_NZ			; ... > 0
		goto	p1PrnX5XSpanSp	; ... = 0: alle 5 Werte senden

	;;	sublw	1-1		; > 1 ?		L - W = 0 - 0...  0 = NC => C
	;;	skip_NC			; ... ja	L - W = 0 - 1...255 = C => NC
	;;	goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Nr zurueck
		sublw	5		; <= 5 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		decf	Para1		; Nr 1...5 ==> 0...4
		movfw	Para2		; Wert zurueck
		sublw	50-1		; > 49 ?	L - W = 49 -  0... 49 = NC => C
		skip_NC			; ... ja	L - W = 49 - 50...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; Wert zurueck
		sublw	210		; <= 210 ?	L - W = 210 -   0...210 = NC => C
		skip_C			; ... ja	L - W = 210 - 211...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_XSpanSp_5	movlw	LOW(pSpanSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para2		; Wert zurueck
		movwf	INDF		; Wert speichern

		movlw	LOW(eepSpanSp)	; Eeprom-Adresse
		addwf	Para1,W		; + Speicher-Nr
		movwf	AddrL
		movfw	Para2		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_XSpanSp_7	call	p1PrnXXSpanSp
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "X" und 1 Wert senden
		; ---------------------------
p1PrnXXSpanSp	movlw	"X"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; 1 Wert senden
		; ---------------------------
p1PrnXSpanSp	movlw	LOW(pSpanSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		goto	p1PrnZif31

		; ---------------------------
		; "X" und alle 5 Werte senden
		; ---------------------------
p1PrnX5XSpanSp	movlw	"X"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden
		movlw	"0"
		call	p1PrnChr	; 0 senden
		call	p1Prn5XSpanSp	; Alle 5 Werte senden
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; Alle 5 Speicher-Werte senden
		; ---------------------------
p1Prn5XSpanSp	movlw	LOW(pSpanSp)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte31

; ===============================================================
; Frequenz Speicher 1...5		Y 1...5 40...200
; ===============================================================
Cmd_YFreqSp	call	p1Such0_9	; Nr suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_7	; ... nicht gefunden: SpeicherNr fehlt

		movwf	Para1		; Nr retten
		call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_YFreqSp_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; Wert retten
		movfw	Para1		; Nr zurueck
		skip_NZ			; ... > 0
		goto	p1PrnX5YFreqSp	; ... = 0: alle 5 Werte senden

	;;	sublw	1-1		; > 1 ?		L - W = 0 - 0...  0 = NC => C
	;;	skip_NC			; ... ja	L - W = 0 - 1...255 = C => NC
	;;	goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Nr zurueck
		sublw	5		; <= 5 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		decf	Para1		; Nr 1...5 ==> 0...4
		movfw	Para2		; Wert zurueck
		sublw	40-1		; > 39 ?	L - W = 39 -  0... 39 = NC => C
		skip_NC			; ... ja	L - W = 39 - 40...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; Wert zurueck
		sublw	200		; <= 200 ?	L - W = 200 -   0...200 = NC => C
		skip_C			; ... ja	L - W = 200 - 201...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_YFreqSp_5	movlw	LOW(pFreqSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para2		; Wert zurueck
		movwf	INDF		; Wert speichern

		movlw	LOW(eepFreqSp)	; Eeprom-Adresse
		addwf	Para1,W		; + Speicher-Nr
		movwf	AddrL
		movfw	Para2		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_YFreqSp_7	call	p1PrnXYFreqSp
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "Y" und 1 Wert senden
		; ---------------------------
p1PrnXYFreqSp	movlw	"Y"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; 1 Wert senden
		; ---------------------------
p1PrnYFreqSp	movlw	LOW(pFreqSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		goto	p1PrnZif30

		; ---------------------------
		; "Y" und alle 5 Werte senden
		; ---------------------------
p1PrnX5YFreqSp	movlw	"Y"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden
		movlw	"0"
		call	p1PrnChr	; 0 senden
		call	p1Prn5YFreqSp	; Alle 5 Werte senden
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; Alle 5 Speicher-Werte senden
		; ---------------------------
p1Prn5YFreqSp	movlw	LOW(pFreqSp)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte30

; ===============================================================
; DutyCycle Speicher 1...5		Z 1...5 10...90
; ===============================================================
Cmd_ZDutySp	call	p1Such0_9	; Nr suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_7	; ... nicht gefunden: SpeicherNr fehlt

		movwf	Para1		; Nr retten
		call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_ZDutySp_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; Wert retten
		movfw	Para1		; Nr zurueck
		skip_NZ			; ... > 0
		goto	p1PrnX5ZDutySp	; ... = 0: alle 5 Werte senden

	;;	sublw	1-1		; > 1 ?		L - W = 0 - 0...  0 = NC => C
	;;	skip_NC			; ... ja	L - W = 0 - 1...255 = C => NC
	;;	goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Nr zurueck
		sublw	5		; <= 5 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		decf	Para1		; Nr 1...5 ==> 0...4
		movfw	Para2		; Wert zurueck
		sublw	10-1		; >  9 ?	L - W = 9 -  0...  9 = NC => C
		skip_NC			; ... ja	L - W = 9 - 10...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; Wert zurueck
		sublw	90		; <= 90 ?	L - W = 90 -  0... 90 = NC => C
		skip_C			; ... ja	L - W = 90 - 91...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Werte speichern
		; ---------------------------
Cmd_ZDutySp_5	movlw	LOW(pDutySp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para2		; Wert zurueck
		movwf	INDF		; Wert speichern

		movlw	LOW(eepDutySp)	; Eeprom-Adresse
		addwf	Para1,W		; + Speicher-Nr
		movwf	AddrL
		movfw	Para2		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_ZDutySp_7	call	p1PrnZZDutySp
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "Z" und 1 Wert senden
		; ---------------------------
p1PrnZZDutySp	movlw	"Z"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; 1 Wert senden
		; ---------------------------
p1PrnZDutySp	movlw	LOW(pDutySp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		goto	p1PrnZif30

		; ---------------------------
		; "Z" und alle 5 Werte senden
		; ---------------------------
p1PrnX5ZDutySp	movlw	"Z"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden
		movlw	"0"
		call	p1PrnChr	; 0 senden
		call	p1Prn5ZDutySp	; Alle 5 Werte senden
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; Alle 5 Speicher-Werte senden
		; ---------------------------
p1Prn5ZDutySp	movlw	LOW(pDutySp)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte30

; ===============================================================
; Taktbetrieb Takt-Zeit Speicher 1...5	T 1...5 10...250
; ===============================================================
Cmd_TaktSp	call	p1Such0_9	; Nr suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_7	; ... nicht gefunden: SpeicherNr fehlt

		movwf	Para1		; Nr retten
		call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_TaktSp_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; Wert retten
		movfw	Para1		; Nr zurueck
		skip_NZ			; ... > 0
		goto	p1PrnX5TaktSp	; ... = 0: alle 5 Werte senden

	;;	sublw	1-1		; > 1 ?		L - W = 0 - 0...  0 = NC => C
	;;	skip_NC			; ... ja	L - W = 0 - 1...255 = C => NC
	;;	goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Nr zurueck
		sublw	5		; <= 5 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		decf	Para1		; Nr 1...5 ==> 0...4
		movfw	Para2		; Wert zurueck
		sublw	10-1		; >  9 ?	L - W = 9 -  0...  9 = NC => C
		skip_NC			; ... ja	L - W = 9 - 10...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; Wert zurueck
		sublw	250		; <= 250 ?	L - W = 250 -   0...250 = NC => C
		skip_C			; ... ja	L - W = 250 - 251...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_TaktSp_5	movlw	LOW(pTaktSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para2		; Wert zurueck
		movwf	INDF		; Wert speichern

		movlw	LOW(eepTaktSp)	; Eeprom-Adresse
		addwf	Para1,W		; + Speicher-Nr
		movwf	AddrL
		movfw	Para2		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_TaktSp_7	call	p1PrnXTaktSp
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "T" und 1 Wert senden
		; ---------------------------
p1PrnXTaktSp	movlw	"T"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; 1 Wert senden
		; ---------------------------
p1PrnTaktSp	movlw	LOW(pTaktSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		goto	p1PrnZif32

		; ---------------------------
		; "T" und alle 5 Werte senden
		; ---------------------------
p1PrnX5TaktSp	movlw	"T"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden
		movlw	"0"
		call	p1PrnChr	; 0 senden
		call	p1Prn5TaktSp	; Alle 5 Werte senden
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; Alle 5 Speicher-Werte senden
		; ---------------------------
p1Prn5TaktSp	movlw	LOW(pTaktSp)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte32

; ===============================================================
; Taktbetrieb Ruhe-Zeit Speicher 1...5	R 1...5 10...250
; ===============================================================
Cmd_RuheSp	call	p1Such0_9	; Nr suchen
		skip_1	F_OK		; ... gefunden
		goto	PrnCmdErr_7	; ... nicht gefunden: SpeicherNr fehlt

		movwf	Para1		; Nr retten
		call	p1Such0_9	; Wert suchen
		skip_1	F_OK		; ... gefunden
		goto	Cmd_RuheSp_7	; ... nicht gefunden: Werte anzeigen

		movwf	Para2		; Wert retten
		movfw	Para1		; Nr zurueck
		skip_NZ			; ... > 0
		goto	p1PrnX5RuheSp	; ... = 0: alle 5 Werte senden

	;;	sublw	1-1		; > 1 ?		L - W = 0 - 0...  0 = NC => C
	;;	skip_NC			; ... ja	L - W = 0 - 1...255 = C => NC
	;;	goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para1		; Nr zurueck
		sublw	5		; <= 5 ?
		skip_C			; ... ja
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		decf	Para1		; Nr 1...5 ==> 0...4
		movfw	Para2		; Wert zurueck
		sublw	10-1		; >  9 ?	L - W = 9 -  0...  9 = NC => C
		skip_NC			; ... ja	L - W = 9 - 10...255 = C => NC
		goto	PrnCmdErr_4	; ... nein: Parameter zu klein

		movfw	Para2		; Wert zurueck
		sublw	250		; <= 250 ?	L - W = 250 -   0...250 = NC => C
		skip_C			; ... ja	L - W = 250 - 251...255 = C => NC
		goto	PrnCmdErr_5	; ... nein: Parameter zu gross

		; ---------------------------
		; Wert speichern
		; ---------------------------
Cmd_RuheSp_5	movlw	LOW(pRuheSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	Para2		; Wert zurueck
		movwf	INDF		; Wert speichern

		movlw	LOW(eepRuheSp)	; Eeprom-Adresse
		addwf	Para1,W		; + Speicher-Nr
		movwf	AddrL
		movfw	Para2		; Wert zurueck
		call	p1WrEeprom	; und im Eeprom speichern

		; ---------------------------
		; Wert senden
		; ---------------------------
Cmd_RuheSp_7	call	p1PrnXRuheSp
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; "R" und 1 Wert senden
		; ---------------------------
p1PrnXRuheSp	movlw	"R"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden

		; ---------------------------
		; 1 Wert senden
		; ---------------------------
p1PrnRuheSp	movlw	LOW(pRuheSp)	; Speicher-Adresse laden
		addwf	Para1,W		; + Speicher-Nr
		movwf	FSR		; fuer indirekte Adressierung
		bcf	STATUS,IRP	; Bank 0 / 1
		movfw	INDF		; Wert laden
		goto	p1PrnZif32

		; ---------------------------
		; "R" und alle 5 Werte senden
		; ---------------------------
p1PrnX5RuheSp	movlw	"R"
		call	p1PrnChr	; Cmd senden
		call	p1PrnSpc	; Leerzeichen senden
		movlw	"0"
		call	p1PrnChr	; 0 senden
		call	p1Prn5RuheSp	; Alle 5 Werte senden
		call	p1PrnCrLf	; CR / LF senden
		goto	Cmd_Fertig

		; ---------------------------
		; Alle 5 Speicher-Werte senden
		; ---------------------------
p1Prn5RuheSp	movlw	LOW(pRuheSp)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte32

		; ---------------------------
		; Alle 5 Analog-Werte senden
		; ---------------------------
p1Prn5Ana	movlw	LOW(aAdw_1)	; Speicher-Adr
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte30

		; ---------------------------
		; Alle 5 Man.-Werte senden
		; ---------------------------
p1Prn5Man	movlw	LOW(mSpan)	; 1. Adresse
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte30

		; ---------------------------
		; Alle 5 Soll-Werte senden
		; ---------------------------
p1Prn5Sol	movlw	LOW(sSpan)	; 1. Adresse
		movwf	Para1
		movlw	5		; 5 Werte
		movwf	zUni
		goto	p1PrnWerte30

	include	"Tattoo_Prn.asm"

; ###############################################################
	org	0x0F80

; ---------------------------------------------------------------
; Sprung zur Cmd-Routine
; ---------------------------------------------------------------
TblCmd		addwf	PCL
		goto	Cmd_AktWahl	; A	Aktuelle Wahl
		goto	PrnCmdErr_2	; B	
		goto	PrnCmdErr_2	; C	
		goto	Cmd_DutyPw	; D	Duty Cycle	Poti-Werte
		goto	PrnCmdErr_2	;* E	Error
		goto	Cmd_FreqPw	; F	Frequenz	Poti-Werte
		goto	Cmd_Geschw	; G	Geschw. Soll-Ist-Annaeherung
		goto	Cmd_Hilfe	; H	Hilfe
		goto	PrnCmdErr_2	; I	
		goto	PrnCmdErr_2	; J
		goto	PrnCmdErr_2	; K	
		goto	PrnCmdErr_2	; L	
		goto	Cmd_ModFk	; M	Modus Fuss-Kontakt
		goto	PrnCmdErr_2	; N	
		goto	PrnCmdErr_2	; O	
		goto	Cmd_Prog	; P	Programm-Version
		goto	PrnCmdErr_2	; Q
		goto	Cmd_RuheSp	; R	Ruhe-Zeit	Speicher 1-5
		goto	Cmd_SpanPw	; S	Spannung	Poti-Werte
		goto	Cmd_TaktSp	; T	Takt-Zeit	Speicher 1-5
		goto	Cmd_UStrom	; U	Ueberstrom-Zeit
		goto	Cmd_VoltR	; V	Volt immer vom Regler
		goto	Cmd_Werte	; W	Werte anzeigen
		goto	Cmd_XSpanSp	; X	Spannung	Speicher 1-5
		goto	Cmd_YFreqSp	; Y	Frequenz	Speicher 1-5
		goto	Cmd_ZDutySp	; Z	DutyCycle	Speicher 1-5

; ###############################################################
; Programmspeicher Page 2 (0x1000...0x17FF)
; ###############################################################
		org	0x1000

p2Reset		nop		; 00h Resetvektor
		goto	P2Init	; 01h Resetvektor
		goto	P2Init	; 02h Resetvektor
		goto	P2Init	; 03h Resetvektor

P2Isr		M_Page0		; auf Rom-Page 0 schalten
		goto	Interrupt

P2Init		M_Page0		; auf Rom-Page 0 schalten
		goto	ProgBeg

; ###############################################################
; Programmspeicher Page 3 (0x1800...0x1FFF)
; ###############################################################
		org	0x1800

TxtSysErr_0	de	"E 00 ; Kein Fehler", 0
TxtSysErr_1	de	"E 01 ; KaltStart", 0
TxtSysErr_2	de	"E 02 ; WarmStart", 0

TxtSysErr_7	de	"E 07 ; TimeOut Eeprom", 0

TxtCmdErr_1	de	"E 21 ; Kommando fehlt", 0
TxtCmdErr_2	de	"E 22 ; Kommando falsch", 0
TxtCmdErr_3	de	"E 23 ; Parameter fehlt / falsch", 0
TxtCmdErr_4	de	"E 24 ; Parameter zu klein", 0
TxtCmdErr_5	de	"E 25 ; Parameter zu gross", 0
TxtCmdErr_6	de	"E 26 ; Bestaetigung fehlt / falsch", 0
TxtCmdErr_7	de	"E 27 ; Speicher-Nr. fehlt", 0
TxtCmdErr_8	de	"E 28 ; Param2 - Param1 < 10", 0

TxtWerte	de	cr, lf
		de	"; WERTE ------------", cr, lf, 0
		de	"; Spannung P von/bis ", 0 , " V" , cr, lf
		de	"; Frequenz P von/bis ", 0 , " Hz", cr, lf
		de	"; DutyCycl P von/bis ", 0 , " %" , cr, lf
		de	"; Spannung  Speicher ", 0 , " V" , cr, lf
		de	"; Frequenz  Speicher ", 0 , " Hz", cr, lf
		de	"; DutyCycle Speicher ", 0 , " %" , cr, lf
		de	"; Taktbetr. Ein-Zeit ", 0 , " s" , cr, lf
		de	"; Taktbetr. Aus-Zeit ", 0 , " s" , cr, lf
		de	"; Ueberstrom-ED      ", 0 , " %" , cr, lf
		de	"; Geschw. Soll-Ist   ", 0 , cr, lf
		de	"; Modus Fuss-Kontakt ", 0 , cr, lf
		de	"; Volt immer regeln    ", 0 , cr, lf
		de	"; Aktuelle Wahl        ", 0 , cr, lf
		de	"; Analog-Werte       ", 0 , cr, lf
		de	"; Manuelle Werte     ", 0 , cr, lf
		de	"; Soll-Werte         ", 0 , cr, lf
		de	"; Programm-Version   ", 0, cr, lf, 0

TxtHilfe	de cr, lf
		de "; S 50-200 60-210 = Spannung  Poti von/bis  [0.1 V]", cr, lf
		de "; F 40-190 50-200 = Frequenz  Poti von/bis  [Hz]", cr, lf
		de "; D 10- 80 20- 90 = DutyCycle Poti von/bis  [%]", cr, lf
		de "; X  1-  5 50-210 = Speicher 1-5  Spannung  [0.1 V]", cr, lf
		de "; Y  1-  5 40-200 = Speicher 1-5  Frequenz  [Hz]", cr, lf
		de "; Z  1-  5 10- 90 = Speicher 1-5  DutyCycle [%]", cr, lf
		de "; T  1-  5 10-250 = Taktbetrieb Takt-Zeit   [0.01 s]", cr, lf
		de "; R  1-  5 10-250 = Taktbetrieb Ruhe-Zeit   [0.01 s]", cr, lf
		de "; U 10- 90        = Ueberstrom-ED *         [%]", cr, lf
		de "; G  0- 20        = Geschw. Soll-Ist-Annaeherung", cr, lf
		de "; M  0-  3        = Modus Fuss-Kontakt", cr, lf
		de "; V  0-  1        = Volt immer regeln", cr, lf
		de "; A  1-  7        = Aktuelle Wahl", cr, lf
		de "; W               = Werte anzeigen", cr, lf
		de "; P               = Programm-Version", cr, lf
		de cr, lf, 0

TxtCVers	de	"P "
TxtCVersX	de	"<", Version, " ", Datum, ">", cr, lf, 0

; ***************************************************************
; Eeprom Vorgaben und Versions-Eintragung
; ***************************************************************
		; -------------------------------------
		; VersionsEintrag ab Eeprom Adr. 0x2100...0x210F (max 16 Byte)
		org	0x2100
		;0123456789ABCDEF
	de	Version
		;01234567
	de	Datum

		; -------------------------------------
		; Konfiguration ab Eeprom-Adr. 0x2120...0x215F
		org	0x2120
EepParam1	; nach Bank 1
eepSpanSp	de	100	; Sp 1	10,0 V	; 0x2120
		de	120	; Sp 2	12,0 V
		de	140	; Sp 3	14,0 V
		de	160	; Sp 4	16,0 V
		de	180	; Sp 5	18,0 V

eepFreqSp	de	 70	; Sp 1	 70 Hz
		de	 80	; Sp 2	 80 Hz
		de	 90	; Sp 3	 90 Hz
		de	100	; Sp 4	100 Hz
		de	110	; Sp 5	110 Hz

eepDutySp	de	 30	; Sp 1	30 %
		de	 40	; Sp 2	40 %
		de	 50	; Sp 3	50 %
		de	 60	; Sp 4	60 %
		de	 70	; Sp 5	70 %

eepTaktSp	de	 60	; Sp 1	600 ms Taktbetrieb Takt-Zeit
		de	 50	; Sp 2	500 ms Taktbetrieb Takt-Zeit
		de	 40	; Sp 3	400 ms Taktbetrieb Takt-Zeit
		de	 30	; Sp 4	300 ms Taktbetrieb Takt-Zeit
		de	 20	; Sp 5	200 ms Taktbetrieb Takt-Zeit

eepRuheSp	de	 60	; Sp 1	600 ms Taktbetrieb Ruhe-Zeit
		de	 50	; Sp 2	500 ms Taktbetrieb Ruhe-Zeit
		de	 40	; Sp 3	400 ms Taktbetrieb Ruhe-Zeit
		de	 30	; Sp 4	300 ms Taktbetrieb Ruhe-Zeit
		de	 20	; Sp 5	200 ms Taktbetrieb Ruhe-Zeit

eepReserve	de	  0	; Sp 1	
		de	  0	; Sp 2	
		de	  0	; Sp 3	
		de	  0	; Sp 4	
		de	  0	; Sp 5	

eepRes1		de	255
eepRes2		de	255
eepSpanVon	de	 60	;  6,0 V	Poti Von
eepSpanBis	de	210	; 21,0 V	Poti Bis
eepFreqVon	de	 50	;  50 Hz	Poti Von
eepFreqBis	de	120	; 120 Hz	Poti Bis
eepDutyVon	de	 10	;  10 %		Poti Von
eepDutyBis	de	 90	;  90 %		Poti Bis

AnzParam1	EQU	$ - EepParam1

		org	0x2150
EepParam2	; nach Bank 0
eepGeschw	de	 10	; Geschw. Soll-Ist-Annaeherung
eepImax		de	 65	; Ueberstrom-ED
eepModFk	de	  0	; Modus Fuss-Kontakt
eepVoltR	de	  0	; Volt aus Speicher

AnzParam2	EQU	$ - EepParam2

	END
