; ***************************************************************
; INITIALISIERUNG
; ***************************************************************
ProgBeg		clrf	STATUS		; Bank 0
		clrf	INTCON		; Alle Interrupts sperren
		M_Page0			; auf Rom-Page 0 schalten

		; -------------------------------------
		; RA0/ANA0	Ana=1	Analog 1
		; RA1/ANA1	Ana=1	Analog 2
		; RA2/ANA2	Ana=1	Analog 3
		; RA3/ANA3	Ana=1	Analog 4
		; RA4/T0CKI	Out=1	CS Digital-Poti
		; RA5/ANA4	Ana=1	Analog 5
		; RA6		---=1
		; RA7		---=1
		; -------------------------------------
		movlw	00111111b	; Alle Ausgaenge setzen,
		iorwf	PORTA		; damit beim Setzen der Tris-Register
					; definierte Zustaende herrschen
		; -------------------------------------
		; RB0/INT	Inp=1	Ueberstrom
		; RB1		Out=0	PWM-Ausgang
		; RB2		Inp=1	Fuss-Taster
		; RB3/PGM	Out=1	LED Blink / Prog PGM
		; RB4		Out=1	LED 8
		; RB5		Out=1	LED Adr 0 
		; RB6/PGC	Out=1	LED Adr 1 / Prog PGC
		; RB7/PGD	Out=1	LED Adr 2 / Prog PGD
		; -------------------------------------
		movlw	10111111b
		movwf	PORTB

		; -------------------------------------
		; RC0/T1	Inp=1	Taster 1
		; RC1/PWM2	Inp=1	Taster 2
		; RC2/CCP1	Inp=1	Taster 3
		; RC3/SCK	Out=0	SPI SCK
		; RC4/SDI	Inp=1	SPI SDI
		; RC5/SDO	Out=0	SPI SDO
		; RC6/TX	(Out)	sio_TxD
		; RC7/RX	(Inp)	sio_RxD
		; -------------------------------------
		movlw	11010111b
		andwf	PORTC
		movlw	00010111b
		iorwf	PORTC

		; -----------------------------------------------
		; STATUS (Bank 0)	Status-Register:
		; 7	Bank Sel IndAdr  0=Bank 0-1 (00-FFh) / 1=Bank 2-3 (100-1FFh)
		; 6-5	Bank Sel	00=Bank 0 ... 11=Bank 3
		; 4	TimeOut		0=WD off / 1=WD on
		; 3	Power Down Bit
		; 2	Z	Zero Bit
		; 1	DC	Digit Carry/Borrow# Bit
		; 0	C	Carry/Borrow# Bit
		;		C wird nur bei ADD, SUB, RLF und RRF veraendert
		;		ACHTUNG !!! Bei SUB ist C und DC invertiert !!!
		;		Berechnung: F/L minus W bzw. F/L plus 2cpl-W
		;		Wenn W >  L/F dann Carry=0
		;		Wenn W <= L/F dann Carry=1
		; -----------------------------------------------

		; -----------------------------------------------
		; USART Receive (Bank 0)
		; 7	SPEN	Serial Port Enable
		; 6	RX9	9-bit Receive Enable
		; 5	SREN	Single Receive Enable
		; 4	CREN	Continuous Receive Enable
		; 3	ADDEN	Address Detect Enable
		; 2	FERR	Framing Error Bit
		; 1	OERR	Overrun Error Bit
		; 0	RX9D	9th bit of received data (Parity)
		; -----------------------------------------------
		movlw	10010000b	; (Bank 0) Receive Status	8 Bits
		movwf	RCSTA		; bit 7 = 1	SPEN (RC7=Rx und RC6=Tx)
					; bit 6 = 1	RX9  8 Bit, 2 Stopbit
					; bit 4 = 1	CREN

		; -----------------------------------------------
		; ADCON0 (Bank 0)	AD Control-Register 0 :
		; 7-6	A/D Conversion Clock	00=Fosc/2 / 01=Fosc/8 / 10=Fosc/32 / 11=RCosci
		; 5-3	Analog Channel		000=AN0 (RA0) ... 111=AN7
		; 2	A/D Conversion bit	0=DONE# / 1=GO
		; 1	-
		; 0	A/D On bit
		; -----------------------------------------------
		movlw	10000101b
		;	10		= Fosc/32
		;	  000		= Analog Channel 0
		;	     1		= GO
		;	       1	= ADON
		movwf	ADCON0

		M_Bank1			; Bank 1
		; -----------------------------------------------
		; ADCON1 (Bank 1)	AD Control-Register 1 :
		; 7	Result Format	8 Bits in ADRESH / 2 Bits in ADRESL(7..6 000000)
		; 6-4	-
		; 3-0	Port Config
		;		an7  an6  an5    AN4  AN3  AN2  AN1  AN0  Ref+  Ref-
		;	Bits	RE2  RE1  RE0    RA5  RA3  RA2  RA1  RA0
		;	0000	 A    A    A      A    A    A    A    A   Vdd   Vss
		;	0001	 A    A    A      A   Ref+  A    A    A   RA3   Vss
		;  **	0010	 D    D    D      A    A    A    A    A   Vdd   Vss
		;	0011	 D    D    D      A   Ref+  A    A    A   RA3   Vss
		;	0100	 D    D    D      D    A    D    A    A   Vdd   Vss
		;	0101	 D    D    D      D   Ref+  D    A    A   RA3   Vss
		;	011x	 D    D    D      D    D    D    D    D   Vdd   Vss
		;	1000	 A    A    A      A   Ref+ Ref-  A    A   RA3   RA2
		;	1001	 D    D    A      A    A    A    A    A   Vdd   Vss
		;	1010	 D    D    A      A   Ref+  A    A    A   RA3   Vss
		;	1011	 D    D    A      A   Ref+ Ref-  A    A   RA3   RA2
		;	1100	 D    D    D      A   Ref+ Ref-  A    A   RA3   RA2
		;	1101	 D    D    D      D   Ref+ Ref-  A    A   RA3   RA2
		;	1110	 D    D    D      D    D    D    D    A   Vdd   Vss
		;	1111	 D    D    D      D   Ref+ Ref-  D    A   RA3   RA2
		; -----------------------------------------------
		movlw	00000010b
		;	0		= Result format = Left
		;	    0010	= Analog RA0-RA3 & RA5
		;			= Digital RA4,RE0-RE2
		movwf	ADCON1

		; -------------------------------------
		; RA0/ANA0	Ana	Analog 1
		; RA1/ANA1	Ana	Analog 2
		; RA2/ANA2	Ana	Analog 3
		; RA3/ANA3	Ana	Analog 4
		; RA4/T0CKI	Out	CS Digital-Poti
		; RA5/ANA4	Ana	Analog 5
		; RA6		---
		; RA7		---
		; -------------------------------------
		movlw	11101111b	; 0=Ausgang / 1=Eingang / unverndert
		movwf	TRISA

		; -------------------------------------
		; RB0/INT	Inp	Ueberstrom
		; RB1		Out	PWM-Ausgang
		; RB2		Inp	Fuss-Taster
		; RB3/PGM	Out	LED Blink / Prog PGM
		; RB4		Out	LED 8
		; RB5		Out	LED Adr 0 
		; RB6/PGC	Out	LED Adr 1 / Prog PGC
		; RB7/PGD	Out	LED Adr 2 / Prog PGD
		; -------------------------------------
		movlw	00000101b	; 0=Ausgang / 1=Eingang
		movwf	TRISB

		; -------------------------------------
		; RC0/T1	Inp	Taster 1
		; RC1/PWM2	Inp	Taster 2
		; RC2/CCP1	Inp	Taster 3
		; RC3/SCL	Out	SCK Digital-Poti
		; RC4/SDI	Inp	SDI Digital-Poti
		; RC5/SDO	Out	SDO (frei)
		; RC6/TX	Out	sio_TxD
		; RC7/RX	Inp	sio_RxD
		; -------------------------------------
		movlw	10010111b	; 0=Ausgang / 1=Eingang / unverndert
		andwf	TRISC

		; -----------------------------------------------
		; USART, Timer, IntEnable (Bank 1)
		; -----------------------------------------------
		; USART Transmit (Bank 1)
		; 7	CSRC	Clock Source Select bit
		; 6	TX9	9-bit Transmit Enable
		; 5	TXEN	Transmit Enable
		; 4	SYNC	USART Mode Select bit
		; 3	-
		; 2	BRGH	High Baud Rate Select bit
		; 1	TRMT	Transmit Shift Register Status bit
		; 0	TX9D	9th bit of transmit data (Parity)
		; -----------------------------------------------
		movlw	00100100b	; (Bank 1) Transmit Status
		movwf	TXSTA		; bit 6 = 1	TX9  8 Bit, 2 Stopbit
					; bit 5 = 1	TXEN Transmitter Freigabe
					; bit 4 = 0	SYNC Asynchronous
					; bit 2 = 1	BRGH High Speed
					; bit 0 = 0	1. Stopbit
		movlw	K_9600		; (Bank 1) BaudrateGenerator
		movwf	SPBRG
		movlw	K_01ms		; (Bank 1) 0,1 ms-Wert fuer Timer 2 setzen
		movwf	PR2
		movlw	00100010b	; (Bank 1) Interrupt enable = 1 / disable = 0
		;	0		; PSIPIE	Parallel Slave Port
		;	 0		; ADIE		A/D Converter
		;	  1		; RCIE		USART Receive
		;	   0		; TXIE		USART Transmit
		;	    0		; SSPIE		Synch Serial Port
		;	     0		; CCP1IE	CCP1
		;	      1		; TMR2IE	Timer 2
		;	       0	; TMR1IE	Timer 1
		movwf	PIE1
		bsf	PCON,NOT_POR	; (Bank 1) PowerOnReset Bit ruecksetzen (=1)

		; -----------------------------------------------
		; OPTION_REG (Bank 1)	Options-Register:
		; 7	PortB pull up	0=enabled / 1=disabled
		; 6	Int Edg Sel	0=fallende / 1=steigende Flanke an RB0/INT
		; 5	T0 Clk Source	0=intern CLKOUT / 1=extern RA4/T0CK
		; 4	T0 Src Edg Sel	0=Low to High / High to Low Increment an RA4/T0CK
		; 3	Prescaler Ass	0=TMR0 / 1=WDT
		; 2-0	Prescaler	000=1:2 ... 111=1:256 TMR0
		;			000=1:1 ... 111=1:128 WDT
		; -----------------------------------------------
		;	    0		; Prescaler Ass	= TMR0
		;	   0		; Flanke	= Low To High
		;	  0		; TMR0-Quelle	= intern CLKOUT
		;	 0		; Interrupt	= fallende Flanke
		;	0		; Pull Up	= enabled
		; -----------------------------------------------
 if Quarz == 16	; 16,000 MHz
TMR0_Presc	EQU	00000111b
		;	     111	; TMR0-Prescaler = 16,00 MHz : 4 = 0,25 ys * 256 = 64 ys
 endif		; -----------------------------------------------
 if Quarz == 12	; 12,000 MHz
TMR0_Presc	EQU	00000111b
		;	     111	; TMR0-Prescaler = 12,00 MHz : 4 = 0,333333 ys * 256 = 85 ys
 endif		; -----------------------------------------------
 if Quarz == 10	; 10,240 MHz
TMR0_Presc	EQU	00000111b
		;	     111	; TMR0-Prescaler = 10,24 MHz : 4 = 0,390625 ys * 256 = 100 ys
 endif		; -----------------------------------------------
 if Quarz == 5	; 4,9152 MHz
TMR0_Presc	EQU	00000110b
		;	     110	; TMR0-Prescaler = 4,9152 MHz : 4 = 0,813802083 ys * 128 = 104 ys
 endif		; -----------------------------------------------
 if Quarz == 4	; 4,096 MHz
TMR0_Presc	EQU	00000101b
		;	     101	; TMR0-Prescaler = 4,096 MHz : 4 = 0,976562 ys * 64 = 62 ys
 endif		; -----------------------------------------------
 if Quarz == 2	; 2,4576 MHz :
TMR0_Presc	EQU	00000101b
		;	     101	; TMR0-Prescaler = 2,4576 MHz : 4 = 1,627042 ys * 64 = 104 ys
 endif		; -----------------------------------------------
		movlw	TMR0_Presc
		movwf	OPTION_REG

		bsf	STATUS,RP0	; Select Bank 1
		; -----------------------------------------------
		; SSPSTAT	SYNC SERIAL PORT STATUS REGISTER (ADDRESS 94h)
		; (Bank 1)
		; -----------------------------------------------
		; 7	SMP	SPI Data Input Sample Phase bit
		;    SPI Master mode:
		;	1 = Input data sampled at end of data output time
		;	0 = Input data sampled at middle of data output time (Microwire)
		;    SPI Slave mode:
		;	SMP must be cleared when SPI is used in Slave mode
		;    I2 C mode:
		;	This bit must be maintained clear
		; 6	CKE	SPI Clock Edge Select bit
		;    SPI mode, CKP = 0:
		;	1 = Data transmitted on rising edge of SCK (Microwire alternate)
		;	0 = Data transmitted on falling edge of SCK
		;    SPI mode, CKP = 1:
		;	1 = Data transmitted on falling edge of SCK (Microwire default)
		; **	0 = Data transmitted on rising edge of SCK
		;    I2 C mode:
		;	This bit must be maintained clear
		; 5	D/A	(I2C mode only)	Data/Address bit
		; 	1 = Indicates that the last byte received or transmitted was data
		;	0 = Indicates that the last byte received or transmitted was address
		; 4 	P	(I2C mode only)	STOP bit
		;    This bit is cleared when the SSP module is disabled, or when the START bit is detected last.
		;    SSPEN is cleared.
		;	1 = Indicates that a STOP bit has been detected last (this bit is 0 on RESET)
		;	0 = STOP bit was not detected last
		; 3 	S	(I2C mode only)	START bit
		;    This bit is cleared when the SSP module is disabled, or when the STOP bit is detected last.
		;    SSPEN is cleared.
		;	1 = Indicates that a START bit has been detected last (this bit is 0 on RESET)
		;	0 = START bit was not detected last
		; 2	R/W	(I2C mode only)	Read/Write bit Information
		;    This bit holds the R/W bit information following the last address match. This bit is only valid from
		;    the address match to the next START bit, STOP bit, or ACK bit.
		;	1 = Read
		;	0 = Write
		; 1	UA	(10-bit I2C mode only)	Update Address bit
		;	1 = Indicates that the user needs to update the address in the SSPADD register
		;	0 = Address does not need to be updated
		; 0	BF	Buffer Full Status bit
		;    Receive (SPI and I2C modes):
		;	1 = Receive complete, SSPBUF is full
		;	0 = Receive not complete, SSPBUF is empty
		;    Transmit (I2C mode only):
		;	1 = Transmit in progress, SSPBUF is full
		;	0 = Transmit complete, SSPBUF is empty
		; -----------------------------------------------
		movlw	10000000b	; SMP=0, CKE=0, BF=0	(fuer AD5290, Dig.-Poti)
		movwf	SSPSTAT

		bcf	STATUS,RP0	; Select Bank 0
		; -----------------------------------------------
		; SSPCON	SYNC SERIAL PORT CONTROL REGISTER (ADDRESS 14h)
		; (Bank 0)
		; -----------------------------------------------
		; 7	WCOL	Write Collision Detect bit
		;		1 = The SSPBUF register is written while it is still transmitting the previous word
		;		(must be cleared in software)
		;		0 = No collision
		; 6	SSPOV	Receive Overflow Indicator bit
		;    In SPI mode:
		;	1 = A new byte is received while the SSPBUF register is still holding the previous data. In case
		;	of overflow, the data in SSPSR is lost. Overflow can only occur in Slave mode. The user
		;	must read the SSPBUF, even if only transmitting data, to avoid setting overflow. In
		;	Master mode, the overflow bit is not set since each new reception (and transmission) is
		;	initiated by writing to the SSPBUF register.
		;	0 = No overflow
		;    In I2 C mode:
		;	1 = A byte is received while the SSPBUF register is still holding the previous byte. SSPOV
		;	is a dont care in Transmit mode. SSPOV must be cleared in software in either mode.
		;	0 = No overflow
		; 5	SSPEN	Synchronous Serial Port Enable bit
		;    In SPI mode:
		;	1 = Enables serial port and configures SCK, SDO, and SDI as serial port pins
		;	0 = Disables serial port and configures these pins as I/O port pins
		;    In I2 C mode:
		;	1 = Enables the serial port and configures the SDA and SCL pins as serial port pins
		;	0 = Disables serial port and configures these pins as I/O port pins
		;    In both modes, when enabled, these pins must be properly configured as input or output.
		; 4	CKP	Clock Polarity Select bit
		;    In SPI mode:
		;	1 = IDLE state for clock is a high level (Microwire default)
		;	0 = IDLE state for clock is a low level (Microwire alternate)
		;    In I2 C mode:
		;	SCK release control
		;	1 = Enable clock
		;	0 = Holds clock low (clock stretch). (Used to ensure data setup time.)
		; 3-0	SSPM3:SSPM0	Synchronous Serial Port Mode Select bits
		;	0000 = SPI Master mode, clock = FOSC/4
		;	0001 = SPI Master mode, clock = FOSC/16
		; ***	0010 = SPI Master mode, clock = FOSC/64
		;	0011 = SPI Master mode, clock = TMR2 output/2
		;	0100 = SPI Slave mode, clock = SCK pin. SS pin control enabled.
		;	0101 = SPI Slave mode, clock = SCK pin. SS pin control disabled. SS can be used as I/O pin.
		;	0110 = I2C Slave mode, 7-bit address
		;	0111 = I2C Slave mode, 10-bit address
		;	1011 = I2C Firmware Controlled Master mode (slave IDLE)
		;	1110 = I2C Slave mode, 7-bit address with START and STOP bit interrupts enabled
		;	1111 = I2C Slave mode, 10-bit address with START and STOP bit interrupts enabled
		; -----------------------------------------------
		; To reset or reconfigure SPI mode,
		; clear bit SSPEN, 
		; re-initialize the SSPCON registers,
		; and then set bit SSPEN.
		bcf	SSPCON,SSPEN	; Bit 5: MSSP Module disable
					; WCOL=0, SSPOV=0, SSPEN=0, CKP=1, SSPM[3..0]= 0010 (FOSC/64)
		movlw	00010010b	; SCK idle high, SPI Master Mode Fosc/64
		movwf	SSPCON
		nop
		bsf	SSPCON,SSPEN	; Bit 5: MSSP Module enable
		bcf	PIR1,SSPIF	; Int-Flag loeschen

		; -----------------------------------------------
		; T1CON (Bank 0)	Timer 1 Control-Register:
		; 7-6	-
		; 5-4	Prescale	11=1:8 / 10=1:4 / 01=1:2 / 00=1:1
		; 3	Osci Enable	1=enabled / 0=shut off
		; 2	Ext Clk Sync
		; 1	Clk Source	1=Extern (RC0) / 0=Intern
		; 0	TimerOnBit	1=On / 0=Off
		;	(Fosc / 4 = 20,000  MHz / 4 = 5,000  MHz = 0,20 ys)
		;	(Fosc / 4 = 16,000  MHz / 4 = 4,000  MHz = 0,25 ys)
		;	(Fosc / 4 = 12,000  MHz / 4 = 3,000  MHz = 0,3333333 ys)
		;	(Fosc / 4 = 10,240  MHz / 4 = 2,560  MHz = 0,3906250 ys)
		;	(Fosc / 4 =  4,9152 MHz / 4 = 1,2288 MHz = 0,8138021 ys)
		;	(Fosc / 4 =  4,096  MHz / 4 = 1,024  MHz = 0,9765625 ys)
		;	(Fosc / 4 =  2,4576 MHz / 4 = 0,6144 MHz = 1,6276042 ys)
		; -----------------------------------------------
 if Quarz == 20	; 20,000 MHz
		movlw	11110001b
		;	  11		; Prescale	= 8 * 0,2      ys = 0,0016 ms
		;	    0		; Osci		= shut off
		;	     0		; Ext Clk Sync	= -
		;	      0		; Clk Source	= Intern
		;	       0	; TimerOn	= Aus
 endif
		; -----------------------------------------------
 if Quarz == 16	; 16,000 MHz
		movlw	11110001b
		;	  11		; Prescale	= 8 * 0,25     ys = 0,002 ms
 endif
 if Quarz == 12	; 12,000 MHz
		movlw	11100001b
		;	  10		; Prescale	= 4 * 0,333333 ys = 0,001333 ms
 endif
		; -----------------------------------------------
 if Quarz == 10	; 10,240 MHz
		movlw	11100001b
		;	  10		; Prescale	= 4 * 0,3906250 ys = 0,001563 ms
 endif
		; -----------------------------------------------
 if Quarz == 5	; 4,9152 MHz
		movlw	11010001b
		;	  01		; Prescale	= 2 * 0,8138021 ys = 0,001628 ms
 endif
		; -----------------------------------------------
 if Quarz == 4	; 4,096 MHz
		movlw	11010001b
		;	  01		; Prescale	= 2 * 0,9765625 ys = 0,001953 ms
 endif
		; -----------------------------------------------
 if Quarz == 2	; 2,4576 MHz :
		movlw	11000001b
		;	  00		; Prescale	= 1 * 1,6276042 ys = 0,001628 ms
 endif
		; -----------------------------------------------
		movwf	T1CON

		; -----------------------------------------------
		; T2CON	(Bank 0)	Timer 2 Control-Register:
		; 7	-
		; 6-3	Postscale	0000=1:1 ... 1111=1:16 Postscale
		; 2	TimerOnBit	1=On / 0=Off
		; 1-0	Prescale	00=1:1 / 01=1:4 / 1x=1:16
		;	(Fosc / 4 = 20,000  MHz / 4 = 5,000  MHz = 0,20 ys)
		;	(Fosc / 4 = 16,000  MHz / 4 = 4,000  MHz = 0,25 ys)
		;	(Fosc / 4 = 12,000  MHz / 4 = 3,000  MHz = 0,3333333 ys)
		;	(Fosc / 4 = 10,240  MHz / 4 = 2,560  MHz = 0,3906250 ys)
		;	(Fosc / 4 =  4,9152 MHz / 4 = 1,2288 MHz = 0,8138021 ys)
		;	(Fosc / 4 =  4,096  MHz / 4 = 1,024  MHz = 0,9765625 ys)
		;	(Fosc / 4 =  2,4576 MHz / 4 = 0,6144 MHz = 1,6276042 ys)
		; Timer 2 / PR2 (8 Bit) wird fuer 0,1 ms-Interrupt benutzt
		; -----------------------------------------------
 if Quarz == 20	; 20,000 MHz
K_9600		EQU	129		; Baudrate = 20,00 MHz / (16 * (129 + 1)) = 9615,384615 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=   4 * 0,2      ys = 0,0008 ms
K_01ms		EQU	125		; PR2-Wert	= 125 * 0,0008   ms = 0,1 ms
 endif		;	 0000		; Postscale	=   1 * 0,1      ms = 0,1 ms
		; -----------------------------------------------
 if Quarz == 16	; 16,000 MHz
K_9600		EQU	103		; Baudrate = 16,00 MHz / (16 * (103 + 1)) = 9615,384615 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=   4 * 0,25     ys = 0,001  ms
K_01ms		EQU	100		; PR2-Wert	= 100 * 0,001    ms = 0,1 ms
 endif		;	 0000		; Postscale	=   1 * 0,1      ms = 0,1 ms
		; -----------------------------------------------
 if Quarz == 12	; 12,000 MHz
K_9600		EQU	77		; Baudrate = 12,00 MHz / (16 * (77 + 1)) = 9615,384615 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=   4 * 0,333333 ys = 0,001333 ms
K_01ms		EQU	 75		; PR2-Wert	=  75 * 0,001333 ms = 0,099975 ms
 endif		;	 0000		; Postscale	=   1 * 0,099975 ms = 0,099975 ms
		; -----------------------------------------------
 if Quarz == 10	; 10,240 MHz
K_9600		EQU	65		; Baudrate = 10,24 MHz / (16 * (65 + 1)) = 9696,969697 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=   4 * 0,390625    ys = 0,0015625 ms
K_01ms		EQU	 64		; PR2-Wert	=  64 * 0,0015625   ms = 0,1 ms
 endif		;	 0000		; Postscale	=   1 * 0,1         ms = 0,1 ms
		; -----------------------------------------------
 if Quarz == 5	; 4,9152 MHz
K_9600		EQU	31		; Baudrate = 4,9152 MHz / (16 * (31 + 1)) = 9600 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=  4 * 0,813802083    ys = 0,003255208332 ms
K_01ms		EQU	 31		; PR2-Wert	= 31 * 0,003255208332 ms = 0,100911458292 ms
 endif		;	 0000		; Postscale	=  1 * 0,100911458292 ms = 0,100911458292 ms
		; -----------------------------------------------
 if Quarz == 4	; 4,096 MHz
K_9600		EQU	25		; Baudrate = 4,096 MHz / (16 * (25 + 1)) = 9846,153846 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=  4 * 0,9765625  ys = 0,00390625 ms
K_01ms		EQU	26		; PR2-Wert	= 26 * 0,00390625 ms = 0,1015625 ms
 endif		;	 0000		; Postscale	=  1 * 0,1015625  ms = 0,1015625 ms
		; -----------------------------------------------
 if Quarz == 2	; 2,4576 MHz :
K_9600		EQU	15		; Baudrate = 2,4576 MHz / (16 * (15 + 1)) = 9600 Baud
		movlw	00000101b
		;	     1		; TimerOn	= Ein
		;	      01	; Prescale	=  4 * 1,627604167    ys = 0,006510416668 ms
K_01ms		EQU	 15		; PR2-Wert	= 15 * 0,006510416668 ms = 0,09765625002 ms
 endif		;	 0000		; Postscale	=  1 * 0,09765625002  ms = 0,09765625002 ms
		; -----------------------------------------------
		movwf	T2CON

		clrf	TMR0		; Timer 0 loeschen
		clrf	TMR1L		; Timer 1 loeschen
		clrf	TMR1H

	;;	bsf	PIR2,EEIF	; Flag Eeprom Write complete setzen
	;;				; notwendig da dieses Flag vor dem Schreiben abgefragt
	;;				; wird und unnoetige Wartezeit verkuerzt

		; ---------------------------
		; Kalt- oder WarmStart ?
		; ---------------------------
		movfw	ramP		; Steht ein P im Ram ?
		sublw	"P"
		skip_Z			; ... ja
		goto	Init_1		; ... nein

		movfw	ramI		; Steht ein I im Ram ?
		sublw	"I"
		skip_Z			; ... ja
		goto	Init_1		; ... nein

		movfw	ramC		; Steht ein C im Ram ?
		sublw	"C"
		skip_Z			; ... ja
		goto	Init_1		; ... nein

		; ---------------------------
		; WarmStart
		; ---------------------------
		movlw	2		; Warmstart
		movwf	Temp0
		goto	Init_2

		; ---------------------------
		; KaltStart: Ram neu initialisieren
		; ---------------------------
Init_1		call	ResRam		; Ram auf 0 setzen

		M_movlf	"P",ramP
		M_movlf	"I",ramI
		M_movlf	"C",ramC

		; ---------------------------
		; Gleich geht's los
		; ---------------------------
Init_2		M_Page1
		call	p1ResCmdPuf	; Cmd-Puffer ruecksetzen
		call	p1EepToRam	; Konfig-Daten aus Eeprom lesen
		M_Page0

		movlw	9
		movwf	z01ms		; = 9
		incf	z1ms		; = 1
		incf	z10ms		; = 1
	;;	incf	z100ms		; = 1

		clrf	zUpdIS

		; ---------------------------
		; Hier geht's los
		; ---------------------------
Init_5
		; -----------------------------------------------
		; INTCON	(Bank 0)
		; -----------------------------------------------
		movlw	11000000b
		;	1		; GIE	Globaler IntEnable
		;	 1		; PEIE	Periphere IntEnable
		;	  0		; T0IE	Timer 0 IntEnable
		;	   0		; INTE	Externer IntEnable (RB0)
		;	    0		; RBIE	Port B Change IntEnable
		;	     0		; T0IF	Timer 0 IntFlag
		;	      0		; INTF	Externer IntFlag (RB0)
		;	       0	; RBIF	Port B Change IntFlag
		movwf	INTCON
	;;	bsf	T1CON,TMR1ON	; Timer 1 Start

		call	SendStart	; Start-Info zur SIO
		movlw	5		; 5 * 10 ms Verzoegerung
		movwf	t10ms
		nop

Init_6		movfw	t10ms		; Verzoegerung abgelaufen ?
		skip_Z			; ... ja
		goto	Init_6

